{codebase.pas (c)Copyright Sequiter Software Inc., 1988-1998.  All rights reserved.}

{********************************************************************}
{ Use The 80x87 Numeric Coprocessor }
{********************************************************************}

{$N+}

{********************************************************************}
{ Define The CodeBase Unit }
{********************************************************************}

unit CodeBase ;

{********************************************************************}
{ Beginning Of Interface Section }
{********************************************************************}

interface

{$ifndef WIN32}
   uses WinCrt, WinTypes ;
{$else}
   uses Windows ;
{$endif}

const

{********************************************************************}
{ Set CBDLL to the specified DLL }
{********************************************************************}

 {$ifdef WIN32}
  CBDLL = 'C4pas32.DLL';      { CodePascal for Windows DLL }
 {$else}
  CBDLL = 'C4pas16';          { CodePascal for Windows DLL }
 {$endif}

 CCVBX = 'CC2PAS.VBX';

{$IFNDEF WM_USER}      {Delphi does not define this on its own}
   WM_USER =           1024;
{$ENDIF}

S4VERSION = 6401 ; { For use by some internal functions }

{*********************************************************************}
{ Field Types }
{*********************************************************************}

r4bin       = integer('B') ; { binary field }
r4str       = integer('C') ; { character field }
r4date      = integer('D') ; { date field }
r4dateTime  = integer('T') ; { datetime field added by rb May 9/00 }
r4float     = integer('F') ; { Floating point field }
r4gen       = integer('G') ; { General field }
r4log       = integer('L') ; { Logical field }
r4memo      = integer('M') ; { Memo field }
r4num       = integer('N') ; { Numeric or Floating Point field }
r4dateDoub  = integer('d') ; { A date is formatted as a C double }
r4numDoub   = integer('n') ; { A numeric value is formatted as a C double }

r4check     = -5 ; { Used for checking the value of CodeBase members }

{********************************************************************}
{ Return Codes }
{********************************************************************}

r4success        =  0 ;
r4same           =  0 ;
r4found          =  1 ;  { Primary Key Match }
r4down           =  1 ;
r4after          =  2 ;
r4complete       =  2 ;
r4eof            =  3 ;
r4bof            =  4 ;
r4entry          =  5 ;  { No index file entry or no record (go) }
r4descending     = 10 ;
r4unique         = 20 ;  { Key is not unique, do not write/append }
r4uniqueContinue = 25 ;  { Key is not unique, write/append anyway }
r4locked         = 50 ;
r4noCreate       = 60 ;  { Could not create file }
r4noOpen         = 70 ;  { Could not open file }
r4noTag          = 80 ;  { DataIndex::seek, with no default tag }
r4terminate      = 90 ;  { no relation match with terminate set }
r4inactive       = 110;
r4active         = 120;
r4authorize      = 140;
r4connected      = 150;
r4logOpen        = 170;

{********************************************************************}
{ Error Codes }
{********************************************************************}

{ General Disk Access Errors }

e4close   = -10 ;
e4create  = -20 ;
e4len     = -30 ;
e4lenSet  = -40 ;
e4lock    = -50 ;
e4open    = -60 ;
e4permiss = -61 ;
e4access  = -62 ;
e4numFiles = -63 ;
e4fileFind = -64 ;
e4instance = -69 ;
e4read    = -70 ;
e4remove  = -80 ;
e4rename  = -90 ;
e4unlock  = -110 ;
e4write   = -120 ;

{ Database Specific Errors }

e4data      = -200 ;
e4fieldName = -210 ;    { Invalid field name }
e4fieldType = -220 ;
e4recordLen = -230 ;
e4append    = -240 ;
e4seek      = -250 ;

{ Index File Specific Errors }

e4entry   = -300 ;    { Tag entry not located }
e4index   = -310 ;
e4tagName = -330 ;
e4unique  = -340 ;    { Key is not unique }
e4tagInfo = -350 ;

{ Expression Evaluation Errors }

e4commaExpected = -400 ;
e4complete      = -410 ;
e4dataName      = -420 ;
e4lengthErr     = -422 ;
e4notConstant   = -425 ;
e4numParms      = -430 ;
e4overflow      = -440 ; { Overflow while evaluating expression }
e4rightMissing  = -450 ;
e4typeSub       = -460 ;
e4unrecFunction = -470 ;
e4unrecOperator = -480 ;
e4unrecValue    = -490 ;
e4unterminated  = -500 ;
e4tagExpr       = -510 ;

{ Optimization Errors }

e4opt        = -610 ;
e4optSuspend = -620 ;
e4optFlush   = -630 ;

{ Relation Errors }

e4relate    = -710 ;
e4lookupErr = -720 ;
e4relateRefer = -730 ;

{ Report Errors  }

e4report      = -810 ;
e4styleCreate = -811 ;
e4styleSelect = -812 ;
e4styleIndex  = -813 ;
e4areaCreate  = -814 ;
e4groupCreate = -815 ;
e4groupExpr   = -816 ;
e4totalCreate = -817 ;
e4objCreate   = -818 ;
e4repWin      = -819 ;
e4repOut      = -820 ;
e4repSave     = -821 ;
e4repRet      = -822 ;
e4repData     = -823 ;

{ Severe Errors }

e4info          = -910 ; { Unexpected information in internal variable }
e4memory        = -920 ; { Out of memory }
e4parm          = -930 ; { Unexpected parameter }
e4parmNull      = -935 ; { NULL input parameter unexpected }
e4demo          = -940 ; { Exceeded maximum record number for demo }
e4result        = -950 ; { Unexpected result }
e4verify        = -960 ; { Structure Verification Failure }
e4struct        = -970 ; {data structure corrupt or not initialized }

e4notIindex  = -1010 ; { S4OFF_INDEX  }
e4notMemo    = -1020 ; { S4OFF_MEMO   }
e4notRename  = -1030 ; { S4NO_RENAME  }
e4notWrite   = -1040 ; { S4OFF_WRITE  }
e4notClipper = -1050 ; { S4CLIPPER    }
e4notLock    = -1060 ; { S4LOCK_HOOK  }
e4notHook    = -1070 ; { S4ERROR_HOOK }

{ Not Supported Errors }
e4notSupported  = -1090 ; { function unsupported }
e4version       = -1095 ; { application/library version mismatch }

{ Memo Errors }
e4memoCorrupt = -1110 ; { memo file corrupt }
e4memoCreate  = -1120 ; { error creating memo file }

{ Transaction Errors }
e4transViolation   = -1200 ;
e4trans            = -1210 ;
e4rollback         = -1220 ;
e4commit           = -1230 ;
e4transAppend      = -1240 ;

{ Communication Errors }
e4corrupt    = -1300 ;
e4connection = -1310 ;
e4socket     = -1312 ;
e4net        = -1330 ;
e4loadlib    = -1340 ;
e4timeOut    = -1350 ;
e4message    = -1360 ;
e4packetLen  = -1370 ;
e4packet     = -1380 ;

{ Miscellaneous Errors }

e4max       = -1400 ;
e4codeBase  = -1410 ;
e4name      = -1420 ;
e4authorize = -1430 ;

{ Server Failure Errors }

e4server = -2100 ;
e4config = -2110 ;
e4cat    = -2120 ;

{ END of errors }

{ internal returns }

relate4filterRecord = 101 ;
relate4doRemove     = 102 ;
relate4skipped      = 104 ;
relate4blank        = 105 ;
relate4skipRec      = 106 ;
relate4terminate    = 107 ;
relate4exact        = 108 ;
relate4scan         = 109 ;
relate4approx       = 110 ;
relate4sortSkip     = 120 ;
relate4sortDone     = 121 ;

{********************************************************************}
{ Error codes for use in error4 functions. }
{********************************************************************}

E4_MESSAG_EXI    = 'EXITING APPLICATION';
E4_ERROR         = 'Error'              ;
E4_MEMORY_ERR    = 'Memory Error'       ;
E4_ERROR_NUM     = 'Error Number'       ;

{ These are equivalent to the values defined in e4defs.h }

E80101 = 80101   ;     { already connected }
E80102 = 80102   ;     { part number out of sequence }

E90101 = 90101   ;     { connection4initUndo }
E90102 = 90102   ;     { socket4initUndo }
E90103 = 90103   ;     { socket4init }
E90104 = 90104   ;     { connection4init }
E90105 = 90105   ;     { connection4setStatus }
E90106 = 90106   ;     { connection4assign }
E90107 = 90107   ;     { socket4connected }
E90108 = 90108   ;     { socket4connect }
E90109 = 90109   ;     { socket4alloc }
E90110 = 90110   ;     { connection4alloc }
E90111 = 90111   ;     { connection4disconnect }
E90112 = 90112   ;     { connection4type }
E90113 = 90113   ;     { connection4status }
E90114 = 90114   ;     { connection4setLen }
E90115 = 90115   ;     { connection4len }
E90116 = 90116   ;     { connection4dataOffset }
E90117 = 90117   ;     { connection4receive }
E90118 = 90118   ;     { socket4receive }
E90119 = 90119   ;     { connection4send }
E90120 = 90120   ;     { connection4clear }
E90121 = 90121   ;     { connection4addData }
E90122 = 90122   ;     { d4socket }
E90123 = 90123   ;     { connection4clientId }
E90124 = 90124   ;     { connection4id }
E90125 = 90125   ;     { connection4serverId }
E90126 = 90126   ;     { connection4readLock }
E90127 = 90127   ;     { connection4enforceLock }


{********************************************************************}
{ other defines }
{********************************************************************}
CP0             = 0;
CP437           = 1;
CP850           = 2;
CP1252          = 3;

LOCK4OFF        = 0;
LOCK4ALL        = 1;
LOCK4DATA       = 2;
LOCK4APPEND     = 10;
LOCK4FILE       = 20;
LOCK4RECORD     = 30;


OPEN4DENY_NONE  = 0;
OPEN4DENY_RW    = 1;
OPEN4DENY_WRITE = 2;

OPT4EXCLUSIVE   = -1;
OPT4OFF         = 0;
OPT4ALL         = 1;

LOG4ALWAYS      = 2;
LOG4ON          = 1;
LOG4TRANS       = 0;

SORT4MACHINE    = 0;
SORT4GENERAL    = 1;

WAIT4EVER       = -1;

{********************************************************************}
{ Client-Server switches }
{********************************************************************}

DEF4SERVER_ID = 'localhost';  { replace with appropriate value }
DEF4PROCESS_ID = '23165'; { replace with appropriate value }
DEF4PROTOCOL = 'S4SOCK.DLL';

{********************************************************************}
{ End of constants }
{********************************************************************}

type

{********************************************************************}
{ Define pointers to common types }
{********************************************************************}

PDouble           = ^Double ;
PLong             = ^Longint ;
PInt              = ^Integer ;
PPChar            = ^PChar ;
PVoid             = PChar ;
PPVoid            = ^PVoid ;
PWord             = ^Word ;
PByte             = ^Byte ;

{ Borland Pascal lacks type COLORREF }
COLORREF4         = Longint;

{ Borland Pascal lacks type HINSTANCE }
HINSTANCE4        = Word;

{********************************************************************}
{ Define CodeBase structure pointers }
{********************************************************************}

AREA4            = pointer ;
CODE4            = pointer ;
DATA4            = pointer ;
EXPR4            = pointer ;
EXPR4CALC        = pointer ;
FIELD4           = pointer ;
GROUP4           = pointer ;
INDEX4           = pointer ;
LIST4            = pointer ;
MEMO4FILE        = pointer ;
OBJ4             = pointer ;
OBJECT4          = pointer ;
OPT4             = pointer ;
RELATE4          = pointer ;
PRELATE4         = ^RELATE4 ;
REPORT4          = pointer ;
STYLE4           = pointer ;
TAG4             = pointer ;
TAG4FILE         = pointer ;
TOTAL4           = pointer ;

{********************************************************************}
{ Declare the FIELD4INFO and TAG4INFO structure }
{********************************************************************}

FIELD4INFO = Record
   name  : PChar;
   {$ifdef WIN32}
      atype : SmallInt ;
   {$else}
      atype : Integer;
   {$endif}
   len   : Word;
   dec   : Word;
  nulls  : Word;
end;

PFIELD4INFO = ^FIELD4INFO;

TAG4INFO = Record
   name       : PChar;
   expression : PChar;
   filter     : PChar;
   {$ifdef WIN32}
      unique     : SmallInt;
   {$else}
      unique     : Integer;
   {$endif}
   descending : Word;
end;

PTAG4INFO = ^TAG4INFO;

{********************************************************************}
{ CODE4 members }

{ not supported : code4hInst, code4hInstVal }
{ not supported : code4hWnd,  code4hWndVal  }


{$ifndef WIN32}

{ use the 'Val' functions to retrieve the currently set value }
{ use other functions to set the value and return old setting }
{********************************************************************}
{ CodeControls VBX supported only from Delphi 1.x }

Function  ctrl4init: CODE4;
Function  ctrl4initP ( p1: HINSTANCE4): CODE4;
Procedure ctrl4initUndo(cb: CODE4);
Procedure ctrl4initVBX(cb: CODE4; HInstance: Word; initialize: integer; p1: PChar; p2: Longint );
Procedure ctrl4initVBXUndo(cb: CODE4; HInstance: HINSTANCE4);

Function  code4allocLow           ( doInit: Integer; p2: PChar; p3: Longint ): CODE4;


{********************************************************************}
{  CODE4 members                                                     }
{********************************************************************}

Function  code4accessMode         ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4autoOpen           ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4codePage           ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4collatingSequence  ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4createTemp         ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4errCreate          ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4errDefaultUnique   ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4errExpr            ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4errFieldName       ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4errGo              ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4errOff             ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4errOpen            ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4errorCode          ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4errRelate          ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4errSkip            ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4errTagName         ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4fileFlush          ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4hInst              ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4hWnd               ( p1 : CODE4; p2 : HWND ) : HWND;
Procedure code4largeOn            ( p1 : CODE4 );
Function  code4lockAttempts       ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4lockAttemptsSingle ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4lockDelay          ( p1 : CODE4; p2 : Word ) : Word;
Function  code4lockEnforce        ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4log                ( p1 : CODE4; p2 : Integer ): Integer;
Function  code4memExpandBlock     ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4memExpandData      ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4memExpandIndex     ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4memExpandLock      ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4memExpandTag       ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4memSizeBlock       ( p1 : CODE4; p2 : Word ) : Word;
Function  code4memSizeBuffer      ( p1 : CODE4; p2 : Word ) : Word;
Function  code4memSizeMemo        ( p1 : CODE4; p2 : Word ) : Word;
Function  code4memSizeMemoExpr    ( p1 : CODE4; p2 : Word ) : Word;
Function  code4memSizeSortBuffer  ( p1 : CODE4; p2 : Word ) : Word;
Function  code4memSizeSortPool    ( p1 : CODE4; p2 : Word ) : Word;
Function  code4memStartBlock      ( p1 : CODE4; p2 : Word ) : Word;
Function  code4memStartLock       ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4memStartMax        ( p1 : CODE4; p2 : Longint ) : Longint;
Function  code4memStartData       ( p1 : CODE4; p2 : Word ) : Word;
Function  code4memStartIndex      ( p1 : CODE4; p2 : Word ) : Word;
Function  code4memStartTag        ( p1 : CODE4; p2 : Word ) : Word;
Function  code4optimize           ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4readLock           ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4readOnly           ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4safety             ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4singleOpen         ( p1 : CODE4; p2 : Integer ) : Integer;
Function  code4timeout            ( p1 : CODE4 ) : Longint ;
Procedure code4timeoutSet         ( p1 : CODE4; p2 : Longint ) ;
Function  code4optimizeWrite      ( p1 : CODE4; p2 : Integer ) : Integer;

{********************************************************************}
{  CODE4 functions                                                   }
{********************************************************************}
Function  code4calcCreate         ( p1: CODE4; p2: EXPR4; p3: PChar ) : EXPR4CALC;
Procedure code4calcReset          ( p1: CODE4 );
Function  code4close              ( p1: CODE4 ) : Integer;
Function  code4connect            ( p1: CODE4; p2: PChar; p3: PChar; p4: PChar; p5: PChar; p6: PChar ) : Integer;
Function  code4data               ( p1: CODE4; p2: PChar ) : DATA4;
Function  code4dateFormat         ( p1: CODE4 ) : PChar ;
Function  code4dateFormatSet      ( p1: CODE4; p2 : PChar ) : Integer;
Procedure code4exit               ( p1: CODE4 );
Function  code4flush              ( p1: CODE4 ) : Integer;
Function  code4indexExtension     ( p1: CODE4 ) : PChar;

Function  code4init               : CODE4;
Function  code4initUndo           ( p1: CODE4 ) : Integer;

Function  code4lock               ( p1: CODE4 ) : Integer;
Procedure code4lockClear          ( p1: CODE4 );
Function  code4lockFileName       ( p1: CODE4 ) : PChar;
Function  code4lockNetworkId      ( p1: CODE4 ) : PChar;
Function  code4lockUserId         ( p1: CODE4 ) : PChar;
Function  code4lockItem           ( p1: CODE4 ) : Longint;
Function  code4logCreate          ( p1: CODE4; p2: PChar; p3: PChar ) : Integer;
Function  code4logFileName        ( p1: CODE4 ) : PChar;
Function  code4logOpen            ( p1: CODE4; p2: PChar; p3: PChar ) : Integer;
Procedure code4logOpenOff         ( p1: CODE4 );
Function  code4optAll             ( p1: CODE4 ) : Integer;
Function  code4optStart           ( p1: CODE4 ) : Integer;
Function  code4optSuspend         ( p1: CODE4 ) : Integer;
Function  code4tranCommit         ( p1: CODE4 ) : Integer;
Function  code4tranRollback       ( p1: CODE4 ) : Integer;
Function  code4tranStart          ( p1: CODE4 ) : Integer;
Function  code4tranStatus         ( p1: CODE4 ) : Integer;
Function  code4unlock             ( p1: CODE4 ) : Integer;
Function  code4unlockAuto         ( p1: CODE4 ) : Integer;
Procedure code4unlockAutoSet      ( p1: CODE4; p2: Integer );

{********************************************************************}
{ Data File Functions }
{********************************************************************}

Function  d4alias           ( p1 : DATA4 ) : PChar;
Procedure d4aliasSet        ( p1 : DATA4; p2 : PChar );
Function  d4append          ( p1 : DATA4 ) : Integer;
Function  d4appendBlank     ( p1 : DATA4 ) : Integer;
Function  d4appendStart     ( p1 : DATA4; p2 : Integer ) : Integer;
Procedure d4blank           ( p1 : DATA4 );
Function  d4bof             ( p1 : DATA4 ) : Integer;
Function  d4bottom          ( p1 : DATA4 ) : Integer;
Function  d4changed         ( p1 : DATA4; p2 : Integer ) : Integer;
Function  d4check           ( p1 : DATA4 ) : Integer;
Function  d4close           ( p1 : DATA4 ) : Integer;
Function  d4create          ( p1 : CODE4; p2 : PChar; p3 : PFIELD4INFO; p4 : PTAG4INFO ) : DATA4;
Procedure d4delete          ( p1 : DATA4 );
Function  d4deleted         ( p1 : DATA4 ) : Integer;
Function  d4eof             ( p1 : DATA4 ) : Integer;
Function  d4field           ( p1 : DATA4; p2 : PChar ) : FIELD4;
Function  d4fieldInfo       ( p1 : DATA4 ) : PFIELD4INFO;
Function  d4fieldJ          ( p1 : DATA4; p2 : Integer ) : FIELD4;
Function  d4fieldNumber     ( p1 : DATA4; p2 : PChar ) : Integer;
Function  d4fileName        ( p1 : DATA4 ) : PChar;
Function  d4flush           ( p1 : DATA4 ) : Integer;
Function  d4freeBlocks      ( p1 : DATA4 ) : Integer;
Function  d4go              ( p1 : DATA4; p2 : Longint ) : Integer;
Function  d4goBof           ( p1 : DATA4 ) : Integer;
Function  d4goEof           ( p1 : DATA4 ) : Integer;
Function  d4index           ( p1 : DATA4; p2 : PChar ) : INDEX4;
Function  d4lock            ( p1 : DATA4; p2 : Longint ) : Integer;
Function  d4lockAdd         ( p1 : DATA4; p2 : Longint ) : Integer;
Function  d4lockAddAll      ( p1 : DATA4 ) : Integer;
Function  d4lockAddAppend   ( p1 : DATA4 ) : Integer;
Function  d4lockAddFile     ( p1 : DATA4 ) : Integer;
Function  d4lockAll         ( p1 : DATA4 ) : Integer;
Function  d4lockAppend      ( p1 : DATA4 ) : Integer;
Function  d4lockFile        ( p1 : DATA4 ) : Integer;
Function  d4log             ( p1 : DATA4; p2: Integer ) : Integer;
Function  d4logStatus       ( p1 : DATA4 ) : Integer;
Function  d4memoCompress    ( p1 : DATA4 ) : Integer;
Function  d4numFields       ( p1 : DATA4 ) : Integer;
Function  d4open            ( p1 : CODE4; p2 : PChar ) : DATA4;
Function  d4openClone       ( p1: DATA4 ) : DATA4;
Function  d4optimize        ( p1 : DATA4; p2 : Integer ) : Integer;
Function  d4optimizeWrite   ( p1 : DATA4; p2 : Integer ) : Integer;
Function  d4pack            ( p1 : DATA4 ) : Integer;
Function  d4position        ( p1 : DATA4 ) : Double;
Function  d4positionSet     ( p1 : DATA4; p2 : Double ) : Integer;
Procedure d4recall          ( p1 : DATA4 );
Function  d4recCount        ( p1 : DATA4 ) : Longint;
Function  d4recNo           ( p1 : DATA4 ) : Longint;
Function  d4record          ( p1 : DATA4 ) : PChar;
Function  d4recWidth        ( p1 : DATA4 ) : Word;
Function  d4refresh         ( p1 : DATA4 ) : Integer;
Function  d4refreshRecord   ( p1 : DATA4 ) : Integer;
Function  d4reindex         ( p1 : DATA4 ) : Integer;
Function  d4remove          ( p1 : DATA4 ) : Integer;
Function  d4seek            ( p1 : DATA4; p2 : PChar ) : Integer;
Function  d4seekDouble      ( p1 : DATA4; p2 : Double ) : Integer;
Function  d4seekN           ( p1 : DATA4; p2 : PChar; p3: Integer ) : Integer;
Function  d4seekNext        ( p1 : DATA4; p2 : PChar ) : Integer;
Function  d4seekNextDouble  ( p1 : DATA4; p2 : Double ) : Integer;
Function  d4seekNextN       ( p1 : DATA4; p2 : PChar; p3: Integer ) : Integer;
Function  d4skip            ( p1 : DATA4; p2 : Longint ) : Integer;
Function  d4tag             ( p1 : DATA4; p2 : PChar ) : TAG4;
Function  d4tagDefault      ( p1 : DATA4 ) : TAG4;
Function  d4tagNext         ( p1 : DATA4; p2 : TAG4 ) : TAG4;
Function  d4tagPrev         ( p1 : DATA4; p2 : TAG4 ) : TAG4;
Function  d4tagSelect       ( p1 : DATA4; p2 : TAG4 ) : Integer;
Function  d4tagSelected     ( p1 : DATA4 ) : TAG4;
Function  d4tagSync         ( p1 : DATA4; p2 : TAG4 ) : Integer;
Function  d4top             ( p1 : DATA4 ) : Integer;
Function  d4unlock          ( p1 : DATA4 ) : Integer;
Function  d4write           ( p1 : DATA4; p2 : Longint ) : Integer;
Function  d4zap             ( p1 : DATA4; p2, p3 : Longint ) : Integer;

{********************************************************************}
{ Date Functions }
{********************************************************************}

Procedure date4assign    ( p1 : PChar; p2 : Longint );
Function  date4cdow      ( p1 : PChar ) : PChar;
Function  date4cmonth    ( p1 : PChar ) : PChar;
Function  date4day       ( p1 : PChar ) : Integer;
Function  date4dow       ( p1 : PChar ) : Integer;
Procedure date4format    ( p1 : PChar; p2 : PChar; p3 : PChar );
Procedure date4init      ( p1 : PChar; p2 : PChar; p3 : PChar );
Function  date4isLeap    ( p1 : PChar ) : Integer;
Function  date4long      ( p1 : PChar ) : Longint;
Function  date4month     ( p1 : PChar ) : Integer;
Procedure date4today     ( p1 : PChar );
Function  date4year      ( p1 : PChar ) : Integer;

{********************************************************************}
{ Error Functions }
{********************************************************************}

Function  error4         ( p1 : CODE4; p2 : Integer; p3 : Longint ) : Integer;
Function  error4describe ( p1 : CODE4; P2 : Integer; p3 : Longint; p4 : PChar; p5 : PChar; p6: PChar) : Integer;
Procedure error4exitTest ( p1 : CODE4 );
Function  error4file     ( p1 : CODE4; p2 : PChar; p3: Integer) : Integer;
Function  error4hook     ( p1 : CODE4; p2 : Integer ) : Integer;
Function  error4set     ( p1 : CODE4; p2 : Integer ): Integer;
Function  error4text     ( p1 : CODE4; P2 : Longint ) : PChar;

{********************************************************************}
{ Expression Evaluation Functions }
{********************************************************************}

Function  expr4data       ( p1 : EXPR4 ) : DATA4;
Function  expr4double     ( p1 : EXPR4 ) : Double;
Procedure expr4free       ( p1 : EXPR4 );
Function  expr4len        ( p1 : EXPR4 ) : Integer;
Function  expr4parse      ( p1 : DATA4; p2 : PChar ) : EXPR4;
Function  expr4source     ( p1 : EXPR4 ) : PChar;
Function  expr4str        ( p1 : EXPR4 ) : PChar;
Function  expr4true       ( p1 : EXPR4 ) : Integer;
Function  expr4type       ( p1 : EXPR4 ) : Integer;

{********************************************************************}
{ Field Functions }
{********************************************************************}

Procedure f4assignDateTime( p1 : FIELD4; p2 : PChar );
Procedure f4assign       ( p1 : FIELD4; p2 : PChar );
Procedure f4assignChar   ( p1 : FIELD4; p2 : Integer );
Procedure f4assignDouble ( p1 : FIELD4; p2 : Double );
Procedure f4assignField  ( p1 : FIELD4; p2 : FIELD4 );
Procedure f4assignInt    ( p1 : FIELD4; p2 : Integer );
Procedure f4assignLong   ( p1 : FIELD4; p2 : Longint );
Procedure f4assignN      ( p1 : FIELD4; p2 : PChar; p3 : Word );
Procedure f4blank        ( p1 : FIELD4 );
Function  f4char         ( p1 : FIELD4 ) : Integer;
Function  f4data         ( p1 : FIELD4 ) : DATA4;
Function  f4decimals     ( p1 : FIELD4 ) : Integer;
Function  f4double       ( p1 : FIELD4 ) : Double;
Function  f4int          ( p1 : FIELD4 ) : Integer;
Function  f4len          ( p1 : FIELD4 ) : Word;
Function  f4long         ( p1 : FIELD4 ) : Longint;

Function  f4memoAssign   ( p1 : FIELD4; p2 : PChar ) : Integer;
Function  f4memoAssignN  ( p1 : FIELD4; p2 : PChar; p3 : Word ) : Integer;
Function  f4memoFree     ( p1 : FIELD4 ) : Integer;
Function  f4memoLen      ( p1 : FIELD4 ) : Word;
Function  f4memoStr      ( p1 : FIELD4 ) : PChar;

Function  f4name         ( p1 : FIELD4 ) : PChar;
Function  f4number       ( p1 : FIELD4 ) : Integer;
Function  f4ptr          ( p1 : FIELD4 ) : PVoid;
Function  f4str          ( p1 : FIELD4 ) : PChar;
Function  f4true         ( p1 : FIELD4 ) : Integer;
Function  f4type         ( p1 : FIELD4 ) : Integer;
Function  f4dateTime     ( p1 : FIELD4 ) : PChar;

{********************************************************************}
{ Index Functions }
{********************************************************************}

Function  i4close     ( p1 : INDEX4 ) : Integer;
Function  i4create    ( p1 : DATA4; p2 : PChar; p3 : PTAG4INFO ) : INDEX4; {0 name -> productn}
Function  i4fileName  ( p1 : INDEX4 ) : PChar;
Function  i4open      ( p1 : DATA4; p2 : PChar ) : INDEX4;
Function  i4reindex   ( p1 : INDEX4 ) : Integer;
Function  i4tag       ( p1 : INDEX4; p2 : PChar ) : TAG4;
Function  i4tagAdd    ( p1 : INDEX4; p2: PTAG4INFO ) : Integer;  { only available for .CDX and .MDX }
Function  i4tagInfo   ( p1 : INDEX4 ) : PTAG4INFO;

{********************************************************************}
{ Relation Functions }
{********************************************************************}

Function  relate4bottom      ( p1 : RELATE4 ) : Integer;
Procedure relate4changed     ( p1 : RELATE4 );
Function  relate4createSlave ( p1 : RELATE4; p2 : DATA4; p3 : PChar; p4 : TAG4 ) : RELATE4;
Function  relate4data        ( p1 : RELATE4 ) : DATA4;
Function  relate4dataTag     ( p1 : RELATE4 ) : TAG4;
Function  relate4doAll       ( p1 : RELATE4 ) : Integer;
Function  relate4doOne       ( p1 : RELATE4 ) : Integer;
Function  relate4eof         ( p1 : RELATE4 ) : Integer;
Function  relate4errorAction ( p1 : RELATE4; p2 : Integer ) : Integer;
Function  relate4free        ( p1 : RELATE4; p2 : Integer ) : Integer;
Function  relate4init        ( p1 : DATA4 ) : RELATE4;
Function  relate4lockAdd     ( p1 : RELATE4 ) : Integer;
Function  relate4master      ( p1 : RELATE4 ) : RELATE4;
Function  relate4masterExpr  ( p1 : RELATE4 ) : PChar;
Function  relate4matchLen    ( p1 : RELATE4; p2 : Integer ) : Integer;
Function  relate4next        ( p1 : PRELATE4 ) : Integer;
Function  relate4optimizeable( p1 : RELATE4 ) : Integer;
Function  relate4querySet    ( p1 : RELATE4; p2 : PChar ) : Integer;
Function  relate4skip        ( p1 : RELATE4; p1 : Longint ) : Integer;
Function  relate4skipEnable  ( p1 : RELATE4; p2 : Integer ) : Integer;
Function  relate4sortSet     ( p1 : RELATE4; p2 : PChar ) : Integer;
Function  relate4top         ( p1 : RELATE4 ) : Integer;
Function  relate4type        ( p1 : RELATE4; p2 : Integer ) : Integer;

{ from CODEREPORTER manual : }

Function  relate4retrieve ( p1 : CODE4  ; p2 : PChar; p3 : Integer; p4 : PChar ) : RELATE4;
Function  relate4save     ( p1 : RELATE4; p2 : PChar; p3 : Integer ) : Integer;

{********************************************************************}
{ TAG4 functions }
{********************************************************************}

Function  t4alias          ( p1 : TAG4 ) : PChar;
Function  t4expr           ( p1 : TAG4 ) : PChar;
Function  t4filter         ( p1 : TAG4 ) : PChar;
Function  t4unique         ( p1 : TAG4 ) : Integer;
Function  t4uniqueSet      ( p1 : TAG4; p2 : Integer ) : Integer;
Function  t4open           ( p1 : DATA4; p2 : PChar ) : TAG4;
Function  t4close          ( p1 : TAG4 ) : Integer;


{********************************************************************}
{ Utility Functions }
{********************************************************************}

Procedure u4free       ( p1 : PVoid );

{********************************************************************}
{ REPORT4 Functions }
{********************************************************************}

Function  report4caption        ( p1 : REPORT4; p2 : PChar ) : Integer;

Function  report4currency       ( p1 : REPORT4; p2 : Char ) : Integer;
Function  report4dateFormat     ( p1 : REPORT4; p2 : PChar ) : Integer;
Function  report4decimal        ( p1 : REPORT4; p2 : Char ) : Integer;
Function  report4do             ( p1 : REPORT4 ) : Integer;
Function  report4doP            ( p1 : REPORT4 ) : Integer;
Procedure report4free           ( p1 : REPORT4; p2 : Integer; p2 : Integer );

Function  report4margins        ( p1 : REPORT4; p2, p3, p4, p5 : Longint; p6 : Integer ) : Integer;
Function  report4pageSize       ( p1 : REPORT4; p2 : Longint; p3 : Longint; p4 : Integer ) : Integer;

Function  report4parent         ( p1 : REPORT4; p2 : HWND ) : Integer;
Procedure report4printerSelect  ( p1 : REPORT4 );

Function  report4querySet       ( p1 : REPORT4; p2 : PChar ) : Integer;
Function  report4relate         ( p1 : REPORT4 ) : RELATE4;
Function  report4retrieve       ( p1 : CODE4; p2 : PChar; p3 : Integer; p4 : PChar ) : REPORT4;
Function  report4screenBreaks   ( p1 : REPORT4; p2 : Integer ) : Integer;
Function  report4save           ( p1 : REPORT4; p2 : PChar; p3 : Integer ) : Integer;
Function  report4separator      ( p1 : REPORT4; p2 : Char ) : Integer;
Function  report4sortSet        ( p1 : REPORT4; p2 : PChar ) : Integer;

Function  report4toScreen       ( p1 : REPORT4; p2 : Integer ) : Integer;

{********************************************************************}
{ Non-User Functions }
{********************************************************************}
Function  c4atol                ( p1 : PChar; p2 : Integer ): Longint;

Function  code4initP            : CODE4;
Function  code4initUndoP        ( p1 : CODE4 ) : Integer;
Function  code4tranStatusP      ( p1: CODE4 ) : Integer;
Procedure code4lockClearP       ( p1: CODE4 );
Function  code4unlockAutoP      ( p1: CODE4 ) : Integer;
Procedure code4unlockAutoSetP   ( p1: CODE4; p2: Integer );

Procedure date4initP            ( p1 : PChar; p2 : PChar; p3 : PChar );

Function  d4logStatusP          ( p1 : DATA4 ) : Integer;
Function d4tagSelectP           ( p1 : DATA4; p2 : TAG4 ) : Integer;
{Function  d4writeLow           ( p1 : DATA4; p2: Longint; p3: Integer ): Integer;}
Function  d4writeP              ( p1 : DATA4; p2 : Longint ) : Integer;

Function  error4default         ( p1 : CODE4; p2 : Integer; p3: Longint ) : Integer;
Function  error4describeDefault ( p1 : CODE4; P2 : Integer; p3 : Longint; p4 : PChar; p5 : PChar; p6: PChar) : Integer;
Function  error4out             ( p1 : CODE4; p2: integer; p3: PChar; p4: PChar; p5: PChar ): Integer;

Function  expr4double2          ( p1 : EXPR4; p2 : PDouble ) : Integer;
Function  expr4parseLow         ( p1 : DATA4; p2: PChar; p2: TAG4FILE ): EXPR4;
Function  expr4dataP            ( p1 : EXPR4 ): DATA4;
Function  expr4lenP             ( p1 : EXPR4 ): Integer;
Function  expr4typeP            ( p1 : EXPR4 ) : Integer;

Function  f4double2             ( p1 : FIELD4; p2: PDouble ) : Integer;
Function  d4position2           ( p1 : DATA4; p2 : PDouble ) : Integer;


Function  relate4dataP          ( p1 : RELATE4 ) : DATA4;
Function  relate4dataTagP       ( p1 : RELATE4 ) : TAG4;
Function  relate4masterP        ( p1 : RELATE4 ): RELATE4;
Function  relate4masterExprP    ( p1 : RELATE4 ): PChar;

Function  t4exprP               ( p1 : TAG4 ) : PChar;
Function  t4filterP             ( p1 : TAG4 ) : PChar;
Function  t4openP               ( p1 : DATA4; p2 : INDEX4; p3 : PChar ) : TAG4;

Procedure u4freeDefault         ( p1 : PVoid );

Function  u4switch              : Longint;

{$else}

{********************************************************************}
{  CODE4 members                                                     }
{********************************************************************}

Function  code4accessMode         ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4autoOpen           ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4codePage           ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4collatingSequence  ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4createTemp         ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4errCreate          ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4errDefaultUnique   ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4errExpr            ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4errFieldName       ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4errGo              ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4errOff             ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4errOpen            ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4errorCode          ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4errRelate          ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4errSkip            ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4errTagName         ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4fileFlush          ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4hInst              ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4hWnd               ( p1 : CODE4; p2 : HWND ) : HWND; stdcall ;
Procedure code4largeOn            ( p1 : CODE4 ); stdcall ;
Function  code4lockAttempts       ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4lockAttemptsSingle ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4lockDelay          ( p1 : CODE4; p2 : Word ) : Word; stdcall ;
Function  code4lockEnforce        ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4log                ( p1 : CODE4; p2 : Integer ): Integer; stdcall ;
Function  code4memExpandBlock     ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4memExpandData      ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4memExpandIndex     ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4memExpandLock      ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4memExpandTag       ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4memSizeBlock       ( p1 : CODE4; p2 : Word ) : Word; stdcall ;
Function  code4memSizeBuffer      ( p1 : CODE4; p2 : Word ) : Word; stdcall ;
Function  code4memSizeMemo        ( p1 : CODE4; p2 : Word ) : Word; stdcall ;
Function  code4memSizeMemoExpr    ( p1 : CODE4; p2 : Word ) : Word; stdcall ;
Function  code4memSizeSortBuffer  ( p1 : CODE4; p2 : Word ) : Word; stdcall ;
Function  code4memSizeSortPool    ( p1 : CODE4; p2 : Word ) : Word; stdcall ;
Function  code4memStartBlock      ( p1 : CODE4; p2 : Word ) : Word; stdcall ;
Function  code4memStartLock       ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4memStartMax        ( p1 : CODE4; p2 : Longint ) : Longint; stdcall ;
Function  code4memStartData       ( p1 : CODE4; p2 : Word ) : Word; stdcall ;
Function  code4memStartIndex      ( p1 : CODE4; p2 : Word ) : Word; stdcall ;
Function  code4memStartTag        ( p1 : CODE4; p2 : Word ) : Word; stdcall ;
Function  code4optimize           ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4readLock           ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4readOnly           ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4safety             ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4singleOpen         ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  code4timeout            ( p1 : CODE4 ) : Longint ; stdcall ;
Procedure code4timeoutSet         ( p1 : CODE4; p2 : Longint ) ; stdcall ;
Function  code4optimizeWrite      ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;

{********************************************************************}
{  CODE4 functions                                                   }
{********************************************************************}
Function  code4calcCreate         ( p1: CODE4; p2: EXPR4; p3: PChar ) : EXPR4CALC; stdcall ;
Procedure code4calcReset          ( p1: CODE4 ); stdcall ;
Function  code4close              ( p1: CODE4 ) : Integer; stdcall ;
Function  code4connect            ( p1: CODE4; p2: PChar; p3: PChar; p4: PChar; p5: PChar; p6: PChar ) : Integer; stdcall ;
Function  code4data               ( p1: CODE4; p2: PChar ) : DATA4; stdcall ;
Function  code4dateFormat         ( p1: CODE4 ) : PChar ; stdcall ;
Function  code4dateFormatSet      ( p1: CODE4; p2 : PChar ) : Integer; stdcall ;
Procedure code4exit               ( p1: CODE4 ); stdcall ;
Function  code4flush              ( p1: CODE4 ) : Integer; stdcall ;
Function  code4indexExtension     ( p1: CODE4 ) : PChar; stdcall ;

Function  code4init               : CODE4; stdcall ;
Function  code4initUndo           ( p1: CODE4 ) : Integer; stdcall ;

Function  code4lock               ( p1: CODE4 ) : Integer; stdcall ;
Procedure code4lockClear          ( p1: CODE4 ); stdcall ;
Function  code4lockFileName       ( p1: CODE4 ) : PChar; stdcall ;
Function  code4lockNetworkId      ( p1: CODE4 ) : PChar; stdcall ;
Function  code4lockUserId         ( p1: CODE4 ) : PChar; stdcall ;
Function  code4lockItem           ( p1: CODE4 ) : Longint; stdcall ;
Function  code4logCreate          ( p1: CODE4; p2: PChar; p3: PChar ) : Integer; stdcall ;
Function  code4logFileName        ( p1: CODE4 ) : PChar; stdcall ;
Function  code4logOpen            ( p1: CODE4; p2: PChar; p3: PChar ) : Integer; stdcall ;
Procedure code4logOpenOff         ( p1: CODE4 ); stdcall ;
Function  code4optAll             ( p1: CODE4 ) : Integer; stdcall ;
Function  code4optStart           ( p1: CODE4 ) : Integer; stdcall ;
Function  code4optSuspend         ( p1: CODE4 ) : Integer; stdcall ;
Function  code4tranCommit         ( p1: CODE4 ) : Integer; stdcall ;
Function  code4tranRollback       ( p1: CODE4 ) : Integer; stdcall ;
Function  code4tranStart          ( p1: CODE4 ) : Integer; stdcall ;
Function  code4tranStatus         ( p1: CODE4 ) : Integer; stdcall ;
Function  code4unlock             ( p1: CODE4 ) : Integer; stdcall ;
Function  code4unlockAuto         ( p1: CODE4 ) : Integer; stdcall ;
Procedure code4unlockAutoSet      ( p1: CODE4; p2: Integer ); stdcall ;

{********************************************************************}
{ Data File Functions }
{********************************************************************}

Function  d4alias           ( p1 : DATA4 ) : PChar; stdcall ;
Procedure d4aliasSet        ( p1 : DATA4; p2 : PChar ); stdcall ;
Function  d4append          ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4appendBlank     ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4appendStart     ( p1 : DATA4; p2 : Integer ) : Integer; stdcall ;
Procedure d4blank           ( p1 : DATA4 ); stdcall ;
Function  d4bof             ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4bottom          ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4changed         ( p1 : DATA4; p2 : Integer ) : Integer; stdcall ;
Function  d4check           ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4close           ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4create          ( p1 : CODE4; p2 : PChar; p3 : PFIELD4INFO; p4 : PTAG4INFO ) : DATA4; stdcall ;
Procedure d4delete          ( p1 : DATA4 ); stdcall ;
Function  d4deleted         ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4eof             ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4field           ( p1 : DATA4; p2 : PChar ) : FIELD4; stdcall ;
Function  d4fieldInfo       ( p1 : DATA4 ) : PFIELD4INFO; stdcall ;
Function  d4fieldJ          ( p1 : DATA4; p2 : Integer ) : FIELD4; stdcall ;
Function  d4fieldNumber     ( p1 : DATA4; p2 : PChar ) : Integer; stdcall ;
Function  d4fileName        ( p1 : DATA4 ) : PChar; stdcall ;
Function  d4flush           ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4freeBlocks      ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4go              ( p1 : DATA4; p2 : Longint ) : Integer; stdcall ;
Function  d4goBof           ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4goEof           ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4index           ( p1 : DATA4; p2 : PChar ) : INDEX4; stdcall ;
Function  d4lock            ( p1 : DATA4; p2 : Longint ) : Integer; stdcall ;
Function  d4lockAdd         ( p1 : DATA4; p2 : Longint ) : Integer; stdcall ;
Function  d4lockAddAll      ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4lockAddAppend   ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4lockAddFile     ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4lockAll         ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4lockAppend      ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4lockFile        ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4log             ( p1 : DATA4; p2: Integer ) : Integer; stdcall ;
Function  d4logStatus       ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4memoCompress    ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4numFields       ( p1 : DATA4 ) : SmallInt; stdcall ;
Function  d4open            ( p1 : CODE4; p2 : PChar ) : DATA4; stdcall stdcall ;
Function  d4openClone       ( p1: DATA4 ) : DATA4; stdcall ;
Function  d4optimize        ( p1 : DATA4; p2 : Integer ) : Integer; stdcall ;
Function  d4optimizeWrite   ( p1 : DATA4; p2 : Integer ) : Integer; stdcall ;
Function  d4pack            ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4position        ( p1 : DATA4 ) : Double; stdcall ;
Function  d4positionSet     ( p1 : DATA4; p2 : Double ) : Integer; stdcall ;
Procedure d4recall          ( p1 : DATA4 ); stdcall ;
Function  d4recCount        ( p1 : DATA4 ) : Longint; stdcall ;
Function  d4recNo           ( p1 : DATA4 ) : Longint; stdcall ;
Function  d4record          ( p1 : DATA4 ) : PChar; stdcall ;
Function  d4recWidth        ( p1 : DATA4 ) : Word; stdcall ;
Function  d4refresh         ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4refreshRecord   ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4reindex         ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4remove          ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4seek            ( p1 : DATA4; p2 : PChar ) : Integer; stdcall ;
Function  d4seekDouble      ( p1 : DATA4; p2 : Double ) : Integer; stdcall ;
Function  d4seekN           ( p1 : DATA4; p2 : PChar; p3: Integer ) : Integer; stdcall ;
Function  d4seekNext        ( p1 : DATA4; p2 : PChar ) : Integer; stdcall ;
Function  d4seekNextDouble  ( p1 : DATA4; p2 : Double ) : Integer; stdcall ;
Function  d4seekNextN       ( p1 : DATA4; p2 : PChar; p3: Integer ) : Integer; stdcall ;
Function  d4skip            ( p1 : DATA4; p2 : Longint ) : Integer; stdcall ;
Function  d4tag             ( p1 : DATA4; p2 : PChar ) : TAG4; stdcall ;
Function  d4tagDefault      ( p1 : DATA4 ) : TAG4; stdcall ;
Function  d4tagNext         ( p1 : DATA4; p2 : TAG4 ) : TAG4; stdcall ;
Function  d4tagPrev         ( p1 : DATA4; p2 : TAG4 ) : TAG4; stdcall ;
Function  d4tagSelect       ( p1 : DATA4; p2 : TAG4 ) : Integer; stdcall ;
Function  d4tagSelected     ( p1 : DATA4 ) : TAG4; stdcall ;
Function  d4tagSync         ( p1 : DATA4; p2 : TAG4 ) : Integer; stdcall ;
Function  d4top             ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4unlock          ( p1 : DATA4 ) : Integer; stdcall ;
Function  d4write           ( p1 : DATA4; p2 : Longint ) : Integer; stdcall ;
Function  d4zap             ( p1 : DATA4; p2, p3 : Longint ) : Integer; stdcall ;

{********************************************************************}
{ Date Functions }
{********************************************************************}

Procedure date4assign    ( p1 : PChar; p2 : Longint ); stdcall ;
Function  date4cdow      ( p1 : PChar ) : PChar; stdcall ;
Function  date4cmonth    ( p1 : PChar ) : PChar; stdcall ;
Function  date4day       ( p1 : PChar ) : Integer; stdcall ;
Function  date4dow       ( p1 : PChar ) : Integer; stdcall ;
Procedure date4format    ( p1 : PChar; p2 : PChar; p3 : PChar ); stdcall ;
Procedure date4init      ( p1 : PChar; p2 : PChar; p3 : PChar ); stdcall ;
Function  date4isLeap    ( p1 : PChar ) : Integer; stdcall ;
Function  date4long      ( p1 : PChar ) : Longint; stdcall ;
Function  date4month     ( p1 : PChar ) : Integer; stdcall ;
Procedure date4today     ( p1 : PChar ); stdcall ;
Function  date4year      ( p1 : PChar ) : Integer; stdcall ;

{********************************************************************}
{ Error Functions }
{********************************************************************}

Function  error4         ( p1 : CODE4; p2 : Integer; p3 : Longint ) : Integer; stdcall ;
Function  error4describe ( p1 : CODE4; P2 : Integer; p3 : Longint; p4 : PChar; p5 : PChar; p6: PChar) : Integer; stdcall ;
Procedure error4exitTest ( p1 : CODE4 ); stdcall ;
Function  error4file     ( p1 : CODE4; p2 : PChar; p3: Integer) : Integer; stdcall ;
Function  error4hook     ( p1 : CODE4; p2 : Integer ) : Integer; stdcall ;
Function  error4set     ( p1 : CODE4; p2 : Integer ): Integer; stdcall ;
Function  error4text     ( p1 : CODE4; P2 : Longint ) : PChar; stdcall ;

{********************************************************************}
{ Expression Evaluation Functions }
{********************************************************************}

Function  expr4data       ( p1 : EXPR4 ) : DATA4; stdcall ;
Function  expr4double     ( p1 : EXPR4 ) : Double; stdcall ;
Procedure expr4free       ( p1 : EXPR4 ); stdcall ;
Function  expr4len        ( p1 : EXPR4 ) : Integer; stdcall ;
Function  expr4parse      ( p1 : DATA4; p2 : PChar ) : EXPR4; stdcall ;
Function  expr4source     ( p1 : EXPR4 ) : PChar; stdcall ;
Function  expr4str        ( p1 : EXPR4 ) : PChar; stdcall ;
Function  expr4true       ( p1 : EXPR4 ) : Integer; stdcall ;
Function  expr4type       ( p1 : EXPR4 ) : Integer; stdcall ;

{********************************************************************}
{ Field Functions }
{********************************************************************}

Procedure f4assignDateTime( p1 : FIELD4; p2 : PChar ); stdcall;
Procedure f4assign       ( p1 : FIELD4; p2 : PChar ); stdcall ;
Procedure f4assignChar   ( p1 : FIELD4; p2 : Integer ); stdcall ;
Procedure f4assignDouble ( p1 : FIELD4; p2 : Double ); stdcall ;
Procedure f4assignField  ( p1 : FIELD4; p2 : FIELD4 ); stdcall ;
Procedure f4assignInt    ( p1 : FIELD4; p2 : Integer ); stdcall ;
Procedure f4assignLong   ( p1 : FIELD4; p2 : Longint ); stdcall ;
Procedure f4assignN      ( p1 : FIELD4; p2 : PChar; p3 : Word ); stdcall ;
Procedure f4blank        ( p1 : FIELD4 ); stdcall ;
Function  f4char         ( p1 : FIELD4 ) : Integer; stdcall ;
Function  f4data         ( p1 : FIELD4 ) : DATA4; stdcall ;
Function  f4decimals     ( p1 : FIELD4 ) : Integer; stdcall ;
Function  f4double       ( p1 : FIELD4 ) : Double; stdcall ;
Function  f4int          ( p1 : FIELD4 ) : Integer; stdcall ;
Function  f4len          ( p1 : FIELD4 ) : Word; stdcall ;
Function  f4long         ( p1 : FIELD4 ) : Longint; stdcall ;

Function  f4memoAssign   ( p1 : FIELD4; p2 : PChar ) : Integer; stdcall ;
Function  f4memoAssignN  ( p1 : FIELD4; p2 : PChar; p3 : Word ) : Integer; stdcall ;
Function  f4memoFree     ( p1 : FIELD4 ) : Integer; stdcall ;
Function  f4memoLen      ( p1 : FIELD4 ) : Integer; stdcall ;
Function  f4memoStr      ( p1 : FIELD4 ) : PChar; stdcall ;

Function  f4name         ( p1 : FIELD4 ) : PChar; stdcall ;
Function  f4number       ( p1 : FIELD4 ) : Integer; stdcall ;
Function  f4ptr          ( p1 : FIELD4 ) : PVoid; stdcall ;
Function  f4str          ( p1 : FIELD4 ) : PChar; stdcall ;
Function  f4true         ( p1 : FIELD4 ) : Integer; stdcall ;
Function  f4type         ( p1 : FIELD4 ) : Integer; stdcall ;
Function  f4dateTime     ( p1 : FIELD4 ) : PChar; stdcall;

{********************************************************************}
{ Index Functions }
{********************************************************************}

Function  i4close     ( p1 : INDEX4 ) : Integer; stdcall ;
Function  i4create    ( p1 : DATA4; p2 : PChar; p3 : PTAG4INFO ) : INDEX4; {0 name -> productn} stdcall ;
Function  i4fileName  ( p1 : INDEX4 ) : PChar; stdcall ;
Function  i4open      ( p1 : DATA4; p2 : PChar ) : INDEX4; stdcall ;
Function  i4reindex   ( p1 : INDEX4 ) : Integer; stdcall ;
Function  i4tag       ( p1 : INDEX4; p2 : PChar ) : TAG4; stdcall ;
Function  i4tagAdd    ( p1 : INDEX4; p2: PTAG4INFO ) : Integer;  { only available for .CDX and .MDX } stdcall ;
Function  i4tagInfo   ( p1 : INDEX4 ) : PTAG4INFO; stdcall ;

{********************************************************************}
{ Relation Functions }
{********************************************************************}

Function  relate4bottom      ( p1 : RELATE4 ) : Integer; stdcall ;
Procedure relate4changed     ( p1 : RELATE4 ); stdcall ;
Function  relate4createSlave ( p1 : RELATE4; p2 : DATA4; p3 : PChar; p4 : TAG4 ) : RELATE4; stdcall ;
Function  relate4data        ( p1 : RELATE4 ) : DATA4; stdcall ;
Function  relate4dataTag     ( p1 : RELATE4 ) : TAG4; stdcall ;
Function  relate4doAll       ( p1 : RELATE4 ) : Integer; stdcall ;
Function  relate4doOne       ( p1 : RELATE4 ) : Integer; stdcall ;
Function  relate4eof         ( p1 : RELATE4 ) : Integer; stdcall ;
Function  relate4errorAction ( p1 : RELATE4; p2 : Integer ) : Integer; stdcall ;
Function  relate4free        ( p1 : RELATE4; p2 : Integer ) : Integer; stdcall ;
Function  relate4init        ( p1 : DATA4 ) : RELATE4; stdcall ;
Function  relate4lockAdd     ( p1 : RELATE4 ) : Integer; stdcall ;
Function  relate4master      ( p1 : RELATE4 ) : RELATE4; stdcall ;
Function  relate4masterExpr  ( p1 : RELATE4 ) : PChar; stdcall ;
Function  relate4matchLen    ( p1 : RELATE4; p2 : Integer ) : Integer; stdcall ;
Function  relate4next        ( p1 : PRELATE4 ) : Integer; stdcall ;
Function  relate4optimizeable( p1 : RELATE4 ) : Integer; stdcall ;
Function  relate4querySet    ( p1 : RELATE4; p2 : PChar ) : Integer; stdcall ;
Function  relate4skip        ( p1 : RELATE4; p2 : Longint ) : Integer; stdcall ;
Function  relate4skipEnable  ( p1 : RELATE4; p2 : Integer ) : Integer; stdcall ;
Function  relate4sortSet     ( p1 : RELATE4; p2 : PChar ) : Integer; stdcall ;
Function  relate4top         ( p1 : RELATE4 ) : Integer; stdcall ;
Function  relate4type        ( p1 : RELATE4; p2 : Integer ) : Integer; stdcall ;

{ from CODEREPORTER manual : }

Function  relate4retrieve ( p1 : CODE4  ; p2 : PChar; p3 : Integer; p4 : PChar ) : RELATE4; stdcall ;
Function  relate4save     ( p1 : RELATE4; p2 : PChar; p3 : Integer ) : Integer; stdcall ;

{********************************************************************}
{ TAG4 functions }
{********************************************************************}

Function  t4alias          ( p1 : TAG4 ) : PChar; stdcall ;
Function  t4expr           ( p1 : TAG4 ) : PChar; stdcall ;
Function  t4filter         ( p1 : TAG4 ) : PChar; stdcall ;
Function  t4unique         ( p1 : TAG4 ) : Integer; stdcall ;
Function  t4uniqueSet      ( p1 : TAG4; p2 : Integer ) : Integer; stdcall ;
Function  t4open           ( p1 : DATA4; p2 : PChar ) : TAG4; stdcall ;
Function  t4close          ( p1 : TAG4 ) : Integer; stdcall ;


{********************************************************************}
{ Utility Functions }
{********************************************************************}

Procedure u4free       ( p1 : PVoid ); stdcall ;

{********************************************************************}
{ REPORT4 Functions }
{********************************************************************}

Function  report4caption        ( p1 : REPORT4; p2 : PChar ) : Integer; stdcall ;

Function  report4currency       ( p1 : REPORT4; p2 : Char ) : Integer; stdcall ;
Function  report4dateFormat     ( p1 : REPORT4; p2 : PChar ) : Integer; stdcall ;
Function  report4decimal        ( p1 : REPORT4; p2 : Char ) : Integer; stdcall ;
Function  report4do             ( p1 : REPORT4 ) : Integer; stdcall ;
Function  report4doP            ( p1 : REPORT4 ): Integer; stdcall ;
Procedure report4free           ( p1 : REPORT4; p2 : Integer; p3 : Integer ); stdcall ;

Function  report4margins        ( p1 : REPORT4; p2, p3, p4, p5 : Longint; p6 : Integer ) : Integer; stdcall ;
Function  report4pageSize       ( p1 : REPORT4; p2 : Longint; p3 : Longint; p4 : Integer ) : Integer; stdcall ;

Function  report4parent         ( p1 : REPORT4; p2 : HWND ) : Integer; stdcall ;
Procedure report4printerSelect  ( p1 : REPORT4 ); stdcall ;

Function  report4querySet       ( p1 : REPORT4; p2 : PChar ) : Integer; stdcall ;
Function  report4relate         ( p1 : REPORT4 ) : RELATE4; stdcall ;
Function  report4retrieve       ( p1 : CODE4; p2 : PChar; p3 : Integer; p4 : PChar ) : REPORT4; stdcall ;
Function  report4screenBreaks   ( p1 : REPORT4; p2 : Integer ) : Integer; stdcall ;
Function  report4save           ( p1 : REPORT4; p2 : PChar; p3 : Integer ) : Integer; stdcall ;
Function  report4separator      ( p1 : REPORT4; p2 : Char ) : Integer; stdcall ;
Function  report4sortSet        ( p1 : REPORT4; p2 : PChar ) : Integer; stdcall ;

Function  report4toScreen       ( p1 : REPORT4; p2 : Integer ) : Integer; stdcall ;

{********************************************************************}
{ Non-User Functions }
{********************************************************************}
Function  c4atol                ( p1 : PChar; p2 : Integer ): Longint; stdcall ;

Function  code4initP            : CODE4; stdcall ;
Function  code4initUndoP        ( p1 : CODE4 ) : Integer; stdcall ;
Procedure code4lockClearP       ( p1: CODE4 );
Function  code4tranStatusP      ( p1: CODE4 ) : Integer; stdcall ;
Function  code4unlockAutoP      ( p1: CODE4 ) : Integer; stdcall ;
Procedure code4unlockAutoSetP   ( p1: CODE4; p2: Integer ); stdcall ;

Procedure date4initP            ( p1 : PChar; p2 : PChar; p3 : PChar ); stdcall ;

Function  d4logStatusP          ( p1 : DATA4 ) : Integer; stdcall ;
Function d4tagSelectP           ( p1 : DATA4; p2 : TAG4 ) : Integer; stdcall ;
{Function  d4writeLow           ( p1 : DATA4; p2: Longint; p3: Integer ): Integer;}
Function  d4writeP              ( p1 : DATA4; p2 : Longint ) : Integer; stdcall ;

Function  error4default         ( p1 : CODE4; p2 : Integer; p3: Longint ) : Integer; stdcall ;
Function  error4describeDefault ( p1:CODE4; P2:Integer; p3:Longint; p4:PChar; p5:PChar; p6:PChar):Integer; stdcall;
Function  error4out             ( p1 : CODE4; p2: integer; p3: PChar; p4: PChar; p5: PChar ): Integer; stdcall ;

Function  expr4double2          ( p1 : EXPR4; p2 : PDouble ) : Integer; stdcall ;
Function  expr4parseLow         ( p1 : DATA4; p2: PChar; p3: TAG4FILE ): EXPR4; stdcall ;
Function  expr4dataP            ( p1 : EXPR4 ): DATA4; stdcall ;
Function  expr4lenP             ( p1 : EXPR4 ): Integer; stdcall ;
Function  expr4typeP            ( p1 : EXPR4 ) : Integer; stdcall ;

Function  f4double2             ( p1 : FIELD4; p2: PDouble ) : Integer; stdcall ;
Function  d4position2           ( p1 : DATA4; p2 : PDouble ) : Integer; stdcall ;

Function  relate4dataP          ( p1 : RELATE4 ) : DATA4; stdcall ;
Function  relate4dataTagP       ( p1 : RELATE4 ) : TAG4; stdcall ;
Function  relate4masterP        ( p1 : RELATE4 ): RELATE4; stdcall ;
Function  relate4masterExprP    ( p1 : RELATE4 ): PChar; stdcall ;

Function  t4exprP               ( p1 : TAG4 ) : PChar; stdcall ;
Function  t4filterP             ( p1 : TAG4 ) : PChar; stdcall ;
Function  t4openP               ( p1 : DATA4; p2 : INDEX4; p3 : pChar ) : TAG4; stdcall ;

Procedure u4freeDefault         ( p1 : PVoid ); stdcall ;

Function  u4switch              : Longint; stdcall ;

{$endif}      {win32}

{********************************************************************}
{ End Of Interface Section }
{********************************************************************}

{********************************************************************}
{ Beginning Of Implementation Section }
{********************************************************************}

implementation

{$ifndef WIN32}
{********************************************************************}
{  CodeControls 2.0 VBX support }
{********************************************************************}

Procedure ctrl4initVBX;            external CBDLL;
Procedure ctrl4initVBXUndo;            external CBDLL;
Function ctrl4initP;                  external CBDLL;
{$endif}

{********************************************************************}
{ CODE4 members }
{********************************************************************}

Function  code4accessMode           ; external CBDLL;
{$ifndef WIN32}
Function  code4allocLow             ; external CBDLL;
{$endif}
Function  code4autoOpen             ; external CBDLL;
Function  code4codePage             ; external CBDLL;
Function  code4collatingSequence    ; external CBDLL;
Function  code4createTemp           ; external CBDLL;
Function  code4errCreate            ; external CBDLL;
Function  code4errDefaultUnique     ; external CBDLL;
Function  code4errExpr              ; external CBDLL;
Function  code4errFieldName         ; external CBDLL;
Function  code4errGo                ; external CBDLL;
Function  code4errOff               ; external CBDLL;
Function  code4errOpen              ; external CBDLL;
Function  code4errorCode            ; external CBDLL;
Function  code4errRelate            ; external CBDLL;
Function  code4errSkip              ; external CBDLL;
Function  code4errTagName           ; external CBDLL;
Function  code4fileFlush            ; external CBDLL;
Function  code4hInst                ; external CBDLL;
Function  code4hWnd                 ; external CBDLL;
Procedure code4largeOn              ; external CBDLL;
Function  code4lockAttempts         ; external CBDLL;
Function  code4lockAttemptsSingle   ; external CBDLL;
Function  code4lockDelay            ; external CBDLL;
Function  code4lockEnforce          ; external CBDLL;
Function  code4log                  ; external CBDLL;
Function  code4memExpandBlock       ; external CBDLL;
Function  code4memExpandData        ; external CBDLL;
Function  code4memExpandIndex       ; external CBDLL;
Function  code4memExpandLock        ; external CBDLL;
Function  code4memExpandTag         ; external CBDLL;
Function  code4memSizeBlock         ; external CBDLL;
Function  code4memSizeBuffer        ; external CBDLL;
Function  code4memSizeMemo          ; external CBDLL;
Function  code4memSizeMemoExpr      ; external CBDLL;
Function  code4memSizeSortBuffer    ; external CBDLL;
Function  code4memSizeSortPool      ; external CBDLL;
Function  code4memStartBlock        ; external CBDLL;
Function  code4memStartLock         ; external CBDLL;
Function  code4memStartMax          ; external CBDLL;
Function  code4memStartData         ; external CBDLL;
Function  code4memStartIndex        ; external CBDLL;
Function  code4memStartTag          ; external CBDLL;
Function  code4optimize             ; external CBDLL;
Function  code4readLock             ; external CBDLL;
Function  code4readOnly             ; external CBDLL;
Function  code4safety               ; external CBDLL;
Function  code4singleOpen           ; external CBDLL;
Function  code4timeout              ; external CBDLL;
Procedure code4timeoutSet           ; external CBDLL;
Function  code4optimizeWrite        ; external CBDLL;

{*********************************************************************}
{ CODE4 functions                                                     }
{*********************************************************************}

Function  code4calcCreate       ; external CBDLL;
Procedure code4calcReset        ; external CBDLL;
Function  code4close            ; external CBDLL;
Function  code4connect          ; external CBDLL;
Function  code4data             ; external CBDLL;
Function  code4dateFormat       ; external CBDLL;
Function  code4dateFormatSet    ; external CBDLL;
Procedure code4exit             ; external CBDLL;
Function  code4flush            ; external CBDLL;
Function  code4indexExtension   ; external CBDLL;
Function  code4lock             ; external CBDLL;
Function  code4lockFileName     ; external CBDLL;
Function  code4lockItem         ; external CBDLL;
Function  code4lockNetworkId    ; external CBDLL;
Function  code4lockUserId       ; external CBDLL;
Function  code4logCreate        ; external CBDLL;
Function  code4logFileName      ; external CBDLL;
Function  code4logOpen          ; external CBDLL;
Procedure code4logOpenOff       ; external CBDLL;
Function  code4optAll           ; external CBDLL;
Function  code4optStart         ; external CBDLL;
Function  code4optSuspend       ; external CBDLL;
Function  code4tranCommit       ; external CBDLL;
Function  code4tranRollback     ; external CBDLL;
Function  code4tranStart        ; external CBDLL;
Function  code4unlock           ; external CBDLL;

{********************************************************************}
{ Data File Functions }
{********************************************************************}

Function  d4alias                   ; external CBDLL;
Procedure d4aliasSet                ; external CBDLL;
Function  d4append                  ; external CBDLL;
Function  d4appendBlank             ; external CBDLL;
Function  d4appendStart             ; external CBDLL;
Procedure d4blank                   ; external CBDLL;
Function  d4bof                     ; external CBDLL;
Function  d4bottom                  ; external CBDLL;
Function  d4changed                 ; external CBDLL;
Function  d4check                   ; external CBDLL;
Function  d4close                   ; external CBDLL;
Function  d4create                  ; external CBDLL;
Procedure d4delete                  ; external CBDLL;
Function  d4deleted                 ; external CBDLL;
Function  d4eof                     ; external CBDLL;
Function  d4field                   ; external CBDLL;
Function  d4fieldInfo               ; external CBDLL;
Function  d4fieldJ                  ; external CBDLL;
Function  d4fieldNumber             ; external CBDLL;
Function  d4fileName                ; external CBDLL;
Function  d4flush                   ; external CBDLL;
Function  d4freeBlocks              ; external CBDLL;
Function  d4go                      ; external CBDLL;
Function  d4goBof                   ; external CBDLL;
Function  d4goEof                   ; external CBDLL;
Function  d4index                   ; external CBDLL;
Function  d4lock                    ; external CBDLL;
Function  d4lockAdd                 ; external CBDLL;
Function  d4lockAddAll              ; external CBDLL;
Function  d4lockAddAppend           ; external CBDLL;
Function  d4lockAddFile             ; external CBDLL;
Function  d4lockAll                 ; external CBDLL;
Function  d4lockAppend              ; external CBDLL;
Function  d4lockFile                ; external CBDLL;
Function  d4log                     ; external CBDLL;
Function  d4memoCompress            ; external CBDLL;
Function  d4numFields               ; external CBDLL;
Function  d4open                    ; external CBDLL;
Function  d4openClone               ; external CBDLL;
Function  d4optimize                ; external CBDLL;
Function  d4optimizeWrite           ; external CBDLL;
Function  d4pack                    ; external CBDLL;
Function  d4positionSet             ; external CBDLL;
Procedure d4recall                  ; external CBDLL;
Function  d4recCount                ; external CBDLL name 'd4recCountDo';
Function  d4recNo                   ; external CBDLL;
Function  d4record                  ; external CBDLL;
Function  d4recWidth                ; external CBDLL;
Function  d4refresh                 ; external CBDLL;
Function  d4refreshRecord           ; external CBDLL;
Function  d4reindex                 ; external CBDLL;
Function  d4remove                  ; external CBDLL;
Function  d4seek                    ; external CBDLL;
Function  d4seekDouble              ; external CBDLL;
Function  d4seekN                   ; external CBDLL;
Function  d4seekNext                ; external CBDLL;
Function  d4seekNextDouble          ; external CBDLL;
Function  d4seekNextN               ; external CBDLL;
Function  d4skip                    ; external CBDLL;
Function  d4tag                     ; external CBDLL;
Function  d4tagDefault              ; external CBDLL;
Function  d4tagNext                 ; external CBDLL;
Function  d4tagPrev                 ; external CBDLL;
{Function  d4tagSelect               ; external CBDLL;}
Function  d4tagSelected             ; external CBDLL;
Function  d4tagSync                 ; external CBDLL;
Function  d4top                     ; external CBDLL;
Function  d4unlock                  ; external CBDLL;
Function  d4zap                     ; external CBDLL;

{********************************************************************}
{ Date Functions }
{********************************************************************}

Procedure date4assign        ; external CBDLL;
Function  date4cdow          ; external CBDLL;
Function  date4cmonth        ; external CBDLL;
Function  date4dow           ; external CBDLL;
Procedure date4format        ; external CBDLL;
Function  date4isLeap        ; external CBDLL;
Function  date4long          ; external CBDLL;
Procedure date4today         ; external CBDLL;

{********************************************************************}
{ Error Functions }
{********************************************************************}

Procedure error4exitTest       ; external CBDLL;
Function  error4file           ; external CBDLL;
Function  error4hook           ; external CBDLL;
Function  error4set            ; external CBDLL;
Function  error4text           ; external CBDLL;

{********************************************************************}
{ Expression Evaluation Functions }
{********************************************************************}

Function  expr4source         ; external CBDLL;
Function  expr4str            ; external CBDLL;
Function  expr4true           ; external CBDLL;

{********************************************************************}
{ Field Functions }
{********************************************************************}
Procedure f4assignDateTime    ; external CBDLL;
Procedure f4assign            ; external CBDLL;
Procedure f4assignChar        ; external CBDLL;
Procedure f4assignDouble      ; external CBDLL;
Procedure f4assignField       ; external CBDLL;
Procedure f4assignInt         ; external CBDLL;
Procedure f4assignLong        ; external CBDLL;
Procedure f4assignN           ; external CBDLL;
Procedure f4blank             ; external CBDLL;
Function  f4char              ; external CBDLL;
Function  f4data              ; external CBDLL;
Function  f4decimals          ; external CBDLL;
Function  f4int               ; external CBDLL;
Function  f4len               ; external CBDLL;
Function  f4long              ; external CBDLL;

Function  f4memoAssign              ; external CBDLL;
Function  f4memoAssignN             ; external CBDLL;
Function  f4memoFree                ; external CBDLL;
Function  f4memoLen                 ; external CBDLL;
Function  f4memoStr                 ; external CBDLL;

Function  f4ptr                     ; external CBDLL;
Function  f4name                    ; external CBDLL;
Function  f4number                  ; external CBDLL;
Function  f4str                     ; external CBDLL;
Function  f4true                    ; external CBDLL;
Function  f4type                    ; external CBDLL;
Function  f4dateTime                ; external CBDLL;

{********************************************************************}
{ Index Functions }
{********************************************************************}

Function  i4close                 ; external CBDLL;
Function  i4create                ; external CBDLL;
Function  i4fileName              ; external CBDLL;
Function  i4open                  ; external CBDLL;
Function  i4reindex               ; external CBDLL;
Function  i4tag                   ; external CBDLL;
Function  i4tagAdd                ; external CBDLL;
Function  i4tagInfo               ; external CBDLL;

{********************************************************************}
{ Relation Functions }
{********************************************************************}

Function  relate4bottom      ; external CBDLL;
Procedure relate4changed     ; external CBDLL;
Function  relate4createSlave ; external CBDLL;
Function  relate4doAll       ; external CBDLL;
Function  relate4doOne       ; external CBDLL;
Function  relate4eof         ; external CBDLL;
Function  relate4errorAction ; external CBDLL;
Function  relate4free        ; external CBDLL;
Function  relate4init        ; external CBDLL;
Function  relate4lockAdd     ; external CBDLL;
Function  relate4matchLen    ; external CBDLL;
Function  relate4next        ; external CBDLL;
Function  relate4optimizeable; external CBDLL;
Function  relate4querySet    ; external CBDLL;
Function  relate4skip        ; external CBDLL;
Function  relate4skipEnable  ; external CBDLL;
Function  relate4sortSet     ; external CBDLL;
Function  relate4top         ; external CBDLL;
Function  relate4type        ; external CBDLL;

{ from CODEREPORTER manual : }

Function  relate4retrieve    ; external CBDLL;
Function  relate4save        ; external CBDLL;


{********************************************************************}
{ Tag Functions }
{********************************************************************}

Function  t4alias                ; external CBDLL;
Function  t4unique               ; external CBDLL;
Function  t4uniqueSet            ; external CBDLL;
Function  t4openP                ; external CBDLL;
Function  t4close                ; external CBDLL;

{********************************************************************}
{ C++ Extension - not supported }
{********************************************************************}

{********************************************************************}
{ CodeReporter Section }
{********************************************************************}

{********************************************************************}
{ REPORT4 Functions }
{********************************************************************}

Function  report4caption            ; external CBDLL;
Function  report4currency           ; external CBDLL;
Function  report4dateFormat         ; external CBDLL;
Function  report4decimal            ; external CBDLL;
Function  report4doP                ; external CBDLL;
Procedure report4free               ; external CBDLL;

Function  report4margins            ; external CBDLL;
Function  report4pageSize           ; external CBDLL;

Function  report4parent             ; external CBDLL;
Procedure report4printerSelect      ; external CBDLL;

Function  report4querySet           ; external CBDLL;
Function  report4relate             ; external CBDLL;
Function  report4retrieve           ; external CBDLL;
Function  report4save               ; external CBDLL;
Function  report4screenBreaks       ; external CBDLL;
Function  report4separator          ; external CBDLL;
Function  report4sortSet            ; external CBDLL;
Function  report4toScreen           ; external CBDLL;
{********************************************************************}
{ Non-User Functions }
{********************************************************************}
Function  c4atol                       ; external CBDLL;

Function  code4initP                   ; external CBDLL;
Function  code4initUndoP               ; external CBDLL;
Procedure code4lockClearP              ; external CBDLL;
Function  code4tranStatusP             ; external CBDLL;
Function  code4unlockAutoP             ; external CBDLL;
Procedure code4unlockAutoSetP          ; external CBDLL;

Procedure date4initP                   ; external CBDLL name 'date4init';

Function  d4logStatusP                 ; external CBDLL;
Function  d4tagSelectP                 ; external CBDLL;
{Function  d4writeLow                   ; external CBDLL;}
Function  d4writeP                     ; external CBDLL;

Function  error4default                ; external CBDLL;
Function  error4describeDefault        ; external CBDLL;
Function  error4out                    ; external CBDLL;

Function  expr4dataP                   ; external CBDLL;
Function  expr4double2                 ; external CBDLL;
Function  expr4lenP                    ; external CBDLL;
Function  expr4parseLow                ; external CBDLL;
Function  expr4typeP                   ; external CBDLL;

Function  f4double2                    ; external CBDLL;
Function  d4position2                  ; external CBDLL;

Function  relate4dataP                 ; external CBDLL;
Function  relate4dataTagP              ; external CBDLL;
Function  relate4masterP               ; external CBDLL;
Function  relate4masterExprP           ; external CBDLL;

Function  t4exprP                      ; external CBDLL;
Function  t4filterP                    ; external CBDLL;

Procedure u4freeDefault                ; external CBDLL;

Function  u4switch                     ; external CBDLL;
{********************************************************************}
{ code4emulator() loads the emulator code.  This includes the floating }
{ point statement which is needed.  Include the $N+ option when compiling. }
{********************************************************************}

Procedure code4emulator;
var
  d : Double;

begin
   d := 1.0;
   if d = 1.0 then Exit;
end;

{********************************************************************}
{ The following functions have been redefined }
{********************************************************************}

{********** code4init **********}

Function code4init : CODE4;

begin
   code4emulator;
   code4init := code4initP;
end;

{********** code4initUndo **********}

Function code4initUndo( p1 : CODE4 ) : Integer;

begin
   code4initUndo := code4initUndoP( p1 );
end;

{************ Non-user Functions ***********}

Procedure code4lockClear( p1: CODE4 );
begin
     code4lockClearP( p1 );
end;

Function  code4tranStatus( p1: CODE4 ) : Integer;
begin
     code4tranStatus := code4tranStatusP( p1 );
end;

Function  code4unlockAuto( p1: CODE4 ) : Integer;
begin
     code4unlockAuto := code4unlockAutoP( p1 );
end;

Procedure code4unlockAutoSet( p1: CODE4; p2: Integer );
begin
     code4unlockAutoSetP( p1, p2 );
end;

{********** date4init ***********}

Procedure date4init ( p1 : PChar; p2 : PChar; p3 : PChar );

begin
   date4initP( p1, p2, p3 ) ;

   { need to put a null character at end of string }
   p1[8] := #0 ;
end;


{********** d4position **********}

Function d4position( p1 : DATA4 ) : Double;

var
   d : Double;

begin
   d4position2( p1, @d );
   d4position := d;
end;


{********** d4tagSelect **********}

Function d4tagSelect( p1 : DATA4; p2 : TAG4 ) : Integer;

begin
   d4tagSelect := d4tagSelectP( p1, p2 );
end;


{********** expr4double **********}

Function expr4double( p1 : EXPR4 ) : Double;

var
   d : Double;

begin
   expr4double2( p1, @d );
   expr4double := d;
end;


Function d4write( p1: DATA4; p2: Longint ): Integer;
begin
     d4write := d4writeP( p1, p2 );
end;

Function  d4logStatus       ( p1 : DATA4 ) : Integer;
begin
     d4logStatus := d4logStatusP( p1 );
end;

Function  date4day ( p1 : PChar ) : Integer;
begin
     date4day :=  Integer ( c4atol( p1 + 6, 2 ) );
end;

Function  date4month ( p1 : PChar ) : Integer;
begin
     date4month := Integer ( c4atol( p1 + 4, 2 ) );
end;

Function  date4year ( p1 : PChar ) : Integer;
begin
     date4year :=  Integer ( c4atol( p1, 4 ) );
end;

Function error4( p1 : CODE4; p2 : Integer; p3 : Longint ) : Integer;
begin
     error4 := error4default( p1, p2,p3 );
end;

Function  error4describe ( p1 : CODE4; p2 : Integer; p3 : Longint; p4 : PChar; p5 : PChar; p6: PChar) : Integer;
begin
     error4describe :=  error4describeDefault ( p1, p2, p3, p4, p5, p6);
end;

Function  expr4data( p1: EXPR4 ): DATA4;
begin
     expr4data := expr4dataP( p1 );
end;

Function  expr4len( p1: EXPR4 ): Integer;
begin
     expr4len := expr4lenP( p1 );
end;

Function  expr4type( p1 : EXPR4 ) : Integer;
begin
        expr4type := expr4typeP( p1 );
end;

{********** f4double **********}

Function f4double( p1 : FIELD4 ) : Double;

var
   d : Double;

begin
   f4double2( p1, @d );
   f4double := d;
end;


Function  relate4data        ( p1 : RELATE4 ) : DATA4;
begin
     relate4data := relate4dataP( p1 );
end;

Function  relate4dataTag     ( p1 : RELATE4 ) : TAG4;
begin
     relate4dataTag := relate4dataTagP( p1 );
end;

Function  relate4master   ( p1 : RELATE4 ): RELATE4;
begin
     relate4master := relate4masterP ( p1 );
end;

Function  relate4masterExpr ( p1 : RELATE4 ): PChar;
begin
     relate4masterExpr := relate4masterExprP( p1 );
end;

Function  report4do ( p1 : REPORT4 ) : Integer;
begin
     report4do := report4doP(p1);
end;
Function  expr4parse( p1 : DATA4; p2 : PChar ) : EXPR4;
begin
     expr4parse := expr4parseLow( p1, p2, nil );
end;

Procedure expr4free( p1: EXPR4 );
begin
     u4free( p1 );
end;

Function t4expr( p1 : TAG4 ) : PChar;
begin
     t4expr := t4exprP( p1 );
end;

Function t4filter( p1 : TAG4 ) : PChar;
begin
     t4filter := t4filterP( p1 );
end;

Function  t4open ( p1 : DATA4; p2 : PChar ) : TAG4;
begin
   t4open := t4openP( p1, nil, p2 );
end;

Procedure u4free( p1 : PVoid );
begin
     u4freeDefault( p1 );
end;


{$ifndef WIN32}
{********** ctrl4init **********}

Function ctrl4init: CODE4;
var
   cb: CODE4;
begin
   cb := code4allocLow( 0, DEF4PROTOCOL, S4VERSION );
   ctrl4initVBX(cb, HInstance, 1, DEF4PROTOCOL, S4VERSION );
   ctrl4init := cb;
end;

{********** ctrl4initUndo **********}

Procedure ctrl4initUndo(cb: CODE4);

begin
   ctrl4initVBXUndo(cb, HInstance );
   u4free(cb);
end;
{$endif}

{********************************************************************}
{ End Of Implementation Section }
{********************************************************************}

{********************************************************************}
{ Beginning Of Initialization Section }
{********************************************************************}

begin

{********************************************************************}
{ End Of Initialization Section }
{********************************************************************}

end.





