unit MessLog;

interface

uses
  Windows, Classes, SysUtils, DateUtils, SyncObjs;

const
  // change this if you want the time/date format on each line to be some other way
  DATEFORMAT = 'hh:nn:ss.zzz dd/mm/yy';

type

  // Class to log messages to a file using the string list class...
  TMessLog = class(TStringList)
  private
    fLogFileName, OpenFile: string;
    CS: TCriticalSection;
    OverWrite: Boolean;
    OnePerDay: boolean;
    FullMonth: boolean;
    LDayOfWeek: integer;
    DidStartM: boolean;
    FirstMsg: boolean;

    procedure iFlush(msg: string = '');
    function myWrite(Fd: TStream; Buffer: string; Count: integer = 0): boolean;
    function GetDtFileName: string;
    constructor StartUp(FileName, StartMsg: string; OverWr: boolean; OnePD: boolean; KeepMonth: boolean; noVer: boolean);

  protected
    //

  public
    DebugOn: boolean;
    // if list gets more items than this # when .Add() called, flush it to the file...
    // (1 would always flush, 0 never (until destroy is called, that is!)
    // default is 250 on .create
    maxItems: integer;

    procedure Flush(msg: string = '');
    function Add(const S: string): integer; override;
    function Add_nf(const S: string): integer;
    function dAdd(const S: string): integer;
    function Format(const Format: string; const Args: array of const): integer;

    constructor Create(FileName: string = ''; OverWr: boolean = False; OnePD: boolean = False; KeepMonth: boolean = False; NoVer: boolean = False); overload;
    constructor Create(FileName: string; StartMsg: string; OverWr: boolean = False; OnePD: boolean = False; KeepMonth: boolean = False; NoVer: boolean = False); overload;

    destructor Destroy; override;
  end;

var
  MessageLog: TMessLog;

function myGetFileVersion(fn: string = ''; Comp: boolean = False): string;
function RPos(const SubStr, Str: string): Integer;

implementation

//  get the file version string of file fn
//  works for Dll's and Exe's
//  returns maj.min.rel.build
function myGetFileVersion(fn: string; Comp: boolean): string;
var
  Size, I, X: DWORD;
  Pc: PChar;
  P: Pointer;
  Version: ^TVsFixedFileInfo;
  fmt: string;
begin
  Result := '';
  if(Length(fn) = 0)then
    fn := ParamStr(0);
  Size := GetFileVersionInfoSize(pChar(fn), I);
  if(Size > 0)then
  begin
    Getmem(pc, Size + 1);
    try
      GetfileVersionInfo(PChar(fn), I, Size, Pc);
      VerQueryValue(Pc, '\\', P, X);
      Version := P;
      if(Comp)then
        fmt := '%.3d.'
      else
        fmt := '%d.';
      Result := Result + Format(fmt, [Version.dwFileVersionMS shr 16]);
      Result := Result + Format(fmt, [Version.dwFileVersionMS and $ffff]);
      Result := Result + Format(fmt, [Version.dwFileVersionLS shr 16]);
      Result := Result + Format(Copy(fmt, 1, Length(fmt) - 1), [Version.dwFileVersionLS and $ffff]);
    except
      Result := '';
    end;
    Freemem(Pc);
  end;
end;

function aGetFileVersion(fn: string = ''; Comp: boolean = False): string;
begin
  if(Length(fn) = 0)then
    fn := ParamStr(0);
  Result := fn +  '   Version - ' + myGetFileVersion(fn, Comp);
end;

procedure myDeleteFile(Filename: string);
begin
{$I-}
 try
  // force ReadOnly files to be replaced anyway...
  FileSetAttr(Filename, FileGetAttr(Filename) and not faReadOnly);
  if(not DeleteFile(Filename))then
  begin
    DeleteFile(Filename + '.del');
    RenameFile(Filename, Filename + '.del');
  end;

 except
  // except nothing!?
 end;
{$I+}
end;

function RPos(const SubStr, Str: string): Integer;
var
  i, l: integer;
begin
  l := Length(SubStr);
  i := Length(Str);
  if((l > 0) and (i > 0) and (i >= l))then
  begin
    i := (i - l) + 1;
    while((i > 0) and ((Copy(Str, i, l) <> SubStr)))do
      Dec(i);
    Result := i;
  end
  else
    Result := 0;
end;

// constructor for Class that will handle Logging messages to a file...
constructor TMessLog.Create(FileName, StartMsg: string; OverWr: boolean; OnePD: boolean; KeepMonth: boolean; noVer: boolean);
begin
  inherited Create;    // must call the ancestors create

  StartUp(FileName, StartMsg, OverWr, OnePD, KeepMonth, noVer);
end;

// overloaded varsion of constructor...
constructor TMessLog.Create(FileName: string; OverWr, OnePD, KeepMonth, noVer: boolean);
begin
  inherited Create;    // must call the ancestors create

  Startup(FileName, 'Start Log', OverWr, OnePD, KeepMonth, noVer);
end;

// cleanup, if any...
destructor TMessLog.Destroy;
begin
  if(DidStartM)then
    Flush('End Log')
  else
    Flush;

  CS.Free;

  inherited Destroy;    { call the Ancestors destroy }
end;

constructor TMessLog.StartUp(FileName, StartMsg: string; OverWr, OnePD, KeepMonth, noVer: boolean);
begin
  inherited Create;    { must call the ancestors create }

  CS := TCriticalSection.Create;

  maxItems := 250;    // default of 250 items before auto-flushing
  DebugOn := False;

  fLogFileName := FileName;
  if(Length(fLogFileName) = 0)then
    fLogFileName := 'Messages.log';

  // get the full path into the file name, so we won't get 'confused'
  // if the program changes the default dir...
  fLogFileName := ExpandFileName(fLogFileName);

{$I-}
  ForceDirectories(ExtractFileDir(fLogFileName));
  // clear any errors...
  IOResult();
{$I+}

  FullMonth := KeepMonth;
  Overwrite := OverWr;
  OnePerDay := OnePD;

  if(OnePerDay)then
    OpenFile := GetDtFileName
  else
  begin
    OpenFile := fLogFileName;
    if(Overwrite and FileExists(OpenFile))then
      myDeleteFile(PChar(OpenFile));
  end;

  FirstMsg := not NoVer;
  DidStartM := Length(StartMsg) > 0;
  if(DidStartM)then
    Add(StartMsg);
end;

function TMessLog.GetDtFileName: string;
var
  ext: string;
begin
  ext := ExtractFileExt(fLogFileName);
  if(FullMonth)then
    LDayOfWeek := DayOfTheMonth(Now)
  else
    LDayOfWeek := DayOfWeek(Now);
  Result := sysutils.Format('%s%d%s', [ChangeFileExt(fLogFileName, ''), LDayOfWeek, ext]);
end;

function TMessLog.Add(const S: string): integer;
begin
  CS.Enter;
 try
  Result := inherited Add(FormatDateTime(DATEFORMAT, Now) + ' - ' + S);

  if(maxItems > 0)then
    if(Count >= maxItems)then
    begin
      iFlush;
      Result := -1;
    end;

 finally
  CS.Leave;
 end;
end;

function TMessLog.Add_nf(const S: string): integer;
begin
  CS.Enter;
 try
  Result := inherited Add(FormatDateTime(DATEFORMAT, Now) + ' - ' + S);
 finally
  CS.Leave;
 end;
end;

function TMessLog.dAdd(const S: string): integer;
begin
  if(DebugOn)then
    Result := Add(S)
  else
    Result := -1;
end;

function TMessLog.Format(const Format: string; const Args: array of const): integer;
begin
 try
  Flush(sysutils.Format(Format, Args));
 except
  on E: Exception do
    try
      // if the error is Cannot open file, don't try Flush(), just Add_nf()
      if(Copy(E.Message, 1, 16) = 'Cannot open file')then
      begin
        Add_nf('TMessLog Format() Exception!');
        Add_nf(' - ' + E.Message + '  ' + Format);
      end
      else
      begin
        Flush('TMessLog Format() Exception!');
        Flush(' - ' + E.Message + '  ' + Format);
      end;
    except
      // Last chance, no exceptions allowed...
    end;
 end;
  Result := -1;
end;

function TMessLog.myWrite(Fd: TStream; Buffer: string; Count: integer): boolean;
begin
  if(Count = 0)then
    Count := Length(Buffer)
  else
    Buffer := Copy(Buffer + StringOfChar(' ', Count), 1, Count);
  Buffer := Buffer + #13#10;
  Result := (Fd.Write(PChar(Buffer)^, Count + 2) = Count + 2);
end;

procedure TMessLog.iFlush(msg: string);
var
//  Fd: TFileStream;
  Fd: THandleStream;
  i, Cnt, nDayOfWeek, f: integer;
begin
  if((Count > 0) or (Length(msg) > 0))then
  begin
    // A 'new' log file each day...
    if(OnePerDay)then
    begin
      if(FullMonth)then
        nDayOfWeek := DayOfTheMonth(Now)
      else
        nDayOfWeek := DayOfWeek(Now);
      if(LDayOfWeek <> nDayOfWeek)then
      begin
        OpenFile := GetDtFileName;
        if(Overwrite and FileExists(OpenFile))then
          myDeleteFile(PChar(OpenFile));
      end;
    end;

{$I-}

    if(not FileExists(OpenFile))then
    begin
      f := FileCreate(OpenFile);
      if(f >= 0)then
      begin
        FirstMsg := True;
        FileClose(f);
      end;
    end;

    // sometimes we fail with 'Cannot open file...' when someone else has it open...
    // so try a couple of times...
    Cnt := 2;
    f := -1;
    while(f < 0)do
    begin
      f := FileOpen(OpenFile, fmOpenReadWrite or fmShareDenyNone);
      if(f >= 0)then
        break;
      IOResult();   // clear the error...

      Dec(Cnt);
      if(Cnt <= 0)then
        break;

      Sleep(87);    // wait a few millisecs...
    end;


    if(f >= 0)then
    begin
      Fd := THandleStream.Create(f);
     try
      Fd.Seek(0, soFromEnd);
      if(FirstMsg)then
        myWrite(Fd, StringOfChar('.', Length(DATEFORMAT)) + ' - ' + aGetFileVersion);
      FirstMsg := False;

      for i := 0 to Count - 1 do
        myWrite(Fd, Strings[i]);

      // add in any message passed to Flush function...
      if(Length(msg) > 0)then
        myWrite(Fd, FormatDateTime(DATEFORMAT, Now) + ' - ' + msg);
     finally
      Fd.Free;
      FileClose(f);
     end;

      // clear out the StringList...
      Clear;
    end;

{$I+}

  end;
end;

procedure TMessLog.Flush(msg: string);
begin
  CS.Enter;
 try

  iFlush(msg);

 finally
  CS.Leave;
 end;
end;

end.
