unit ODBCDef;
{
    ODBC Definition Unit, for the ODBC Class.

    This file defines the internally defined constants inside the ODBC32.DLL
    it uses.  Not all of the constants are here, nor are all the ODBC API
    functions defined in ODBC.pas for use by the class have been defined for
    ODBC32.DLL, just the stuff that is used got translated here.

    Created by KRS 6/14/1999.
}
interface

type
  SQLHANDLE = Pointer;
  SQLHENV = SQLHANDLE;
  SQLHDBC = SQLHANDLE;
  SQLHSTMT = SQLHANDLE;

const
  SQL_NULL_HENV = SQLHANDLE(0);

  // Handle Type identifiers.
  SQL_HANDLE_ENV   = 1;
  SQL_HANDLE_DBC   = 2;
  SQL_HANDLE_STMT  = 3;
  SQL_HANDLE_DESC  = 4;

  // Standard Return values for ODBC32.DLL functions.
  SQL_INVALID_HANDLE    = -2;
  SQL_ERROR             = -1;
  SQL_SUCCESS           = 0;
  SQL_SUCCESS_WITH_INFO = 1;
  SQL_STILL_EXECUTING   = 2;
  SQL_NEED_DATA         = 99;
  SQL_NO_DATA           = 100;

  // Message Length
  SQL_MAX_MESSAGE_LENGTH = 512;
  
  // The one Environment Handle Attribute I set and it's values.
  SQL_ATTR_ODBC_VERSION = 200;
    SQL_OV_ODBC2 = 2;
    SQL_OV_ODBC3 = 3;

  SQL_ATTR_CONNECTION_POOLING = 201;
    SQL_CP_OFF              = 0;
    SQL_CP_ONE_PER_DRIVER   = 1;
    SQL_CP_ONE_PER_HENV     = 2;
    SQL_CP_DEFAULT          = SQL_CP_OFF;

  SQL_FALSE   = 0;
  SQL_TRUE    = 1;

  // Standard Cursor Movements allowed by SQLFetchScroll and SQLSetPos.
  SQL_FETCH_NEXT        = 1;
  SQL_FETCH_FIRST       = 2;
  SQL_FETCH_LAST        = 3;
  SQL_FETCH_PRIOR       = 4;
  SQL_FETCH_ABSOLUTE    = 5;
  SQL_FETCH_RELATIVE    = 6;

  // special length/indicator values
  SQL_NULL_DATA         = (-1);
  SQL_DATA_AT_EXEC      = (-2);

  { Standard Data Type constants used to describe parameters being sent to
    functions in ODBC32.DLL. }
  SQL_NTS          = -3;     // Flag for a null-terminate string.
  SQL_IS_POINTER   = -4;
  SQL_IS_UINTEGER  = -5;
  SQL_IS_INTEGER   = -6;
  SQL_IS_USMALLINT = -7;
  SQL_IS_SMALLINT  = -8;

  SQL_LEN_BINARY_ATTR_OFFSET  =  (-100);
  // macro
  function SQL_LEN_BINARY_ATTR(length: integer): integer;

const

  { Standard Data Type constants used in return values use by functions in
    ODBC32.DLL. }

  SQL_GUID              = (-11);
  SQL_WTEXT             = (-10);
  SQL_WVARCHAR          = (-9);
  SQL_WCHAR             = (-8);
  SQL_BIT               = (-7);
  SQL_TINYINT           = (-6);
  SQL_BIGINT            = (-5);
  SQL_IMAGE             = (-4);
  SQL_VARBINARY         = (-3);
  SQL_BINARY            = (-2);
  SQL_LONGVARCHAR       = (-1);


  SQL_UNKNOWN_TYPE      = 0;
  SQL_CHAR              = 1;
  SQL_NUMERIC           = 2;
  SQL_DECIMAL           = 3;
  SQL_INTEGER           = 4;
  SQL_SMALLINT          = 5;
  SQL_FLOAT             = 6;
  SQL_REAL              = 7;
  SQL_DOUBLE            = 8;
  SQL_DATETIME          = 9;
  SQL_DATE              = 9;
  SQL_TIME              = 10;
  SQL_TIMESTAMP         = 11;
  SQL_VARCHAR           = 12;
  SQL_TYPE_DATE         = 91;
  SQL_TYPE_TIME         = 92;
  SQL_TYPE_TIMESTAMP    = 93;

  // Used in the SQLBindParamater()
  SQL_PARAM_TYPE_UNKNOWN          = 0;
  SQL_PARAM_INPUT                 = 1;
  SQL_PARAM_INPUT_OUTPUT          = 2;
  SQL_RESULT_COL                  = 3;
  SQL_PARAM_OUTPUT                = 4;
  SQL_RETURN_VALUE                = 5;

  // DataTypes used in SQLBindParameter...
  SQL_C_CHAR    = SQL_CHAR;
  SQL_C_LONG    = SQL_INTEGER;
  SQL_C_SHORT   = SQL_SMALLINT;
  SQL_C_FLOAT   = SQL_REAL;
  SQL_C_DOUBLE  = SQL_DOUBLE;
  SQL_C_NUMERIC  =  SQL_NUMERIC;
  SQL_C_DEFAULT = 99;

  SQL_SIGNED_OFFSET       = (-20);
  SQL_UNSIGNED_OFFSET     = (-22);

  SQL_C_DATE      = SQL_DATE;
  SQL_C_TIME      = SQL_TIME;
  SQL_C_TIMESTAMP = SQL_TIMESTAMP;
(*
  SQL_C_TYPE_DATE          SQL_TYPE_DATE
  SQL_C_TYPE_TIME          SQL_TYPE_TIME
  SQL_C_TYPE_TIMESTAMP      SQL_TYPE_TIMESTAMP
  SQL_C_INTERVAL_YEAR        SQL_INTERVAL_YEAR
  SQL_C_INTERVAL_MONTH      SQL_INTERVAL_MONTH
  SQL_C_INTERVAL_DAY        SQL_INTERVAL_DAY
  SQL_C_INTERVAL_HOUR        SQL_INTERVAL_HOUR
  SQL_C_INTERVAL_MINUTE      SQL_INTERVAL_MINUTE
  SQL_C_INTERVAL_SECOND      SQL_INTERVAL_SECOND
  SQL_C_INTERVAL_YEAR_TO_MONTH  SQL_INTERVAL_YEAR_TO_MONTH
  SQL_C_INTERVAL_DAY_TO_HOUR    SQL_INTERVAL_DAY_TO_HOUR
  SQL_C_INTERVAL_DAY_TO_MINUTE  SQL_INTERVAL_DAY_TO_MINUTE
  SQL_C_INTERVAL_DAY_TO_SECOND  SQL_INTERVAL_DAY_TO_SECOND
  SQL_C_INTERVAL_HOUR_TO_MINUTE  SQL_INTERVAL_HOUR_TO_MINUTE
  SQL_C_INTERVAL_HOUR_TO_SECOND  SQL_INTERVAL_HOUR_TO_SECOND
  SQL_C_INTERVAL_MINUTE_TO_SECOND  SQL_INTERVAL_MINUTE_TO_SECOND
*)
  SQL_C_SBIGINT  = (SQL_BIGINT+SQL_SIGNED_OFFSET);
  SQL_C_UBIGINT  = (SQL_BIGINT+SQL_UNSIGNED_OFFSET);

  SQL_C_BINARY     = SQL_BINARY;
  SQL_C_BIT        = SQL_BIT;
  SQL_C_TINYINT    = SQL_TINYINT;
  SQL_C_SLONG      = (SQL_C_LONG+SQL_SIGNED_OFFSET);
  SQL_C_SSHORT     = (SQL_C_SHORT+SQL_SIGNED_OFFSET);
  SQL_C_STINYINT   = (SQL_TINYINT+SQL_SIGNED_OFFSET);
  SQL_C_ULONG      = (SQL_C_LONG+SQL_UNSIGNED_OFFSET);
  SQL_C_USHORT     = (SQL_C_SHORT+SQL_UNSIGNED_OFFSET);
  SQL_C_UTINYINT   = (SQL_TINYINT+SQL_UNSIGNED_OFFSET);
  SQL_C_BOOKMARK   = SQL_C_ULONG;

  // These two sets of constants are used by SQLSetPos in ODBC32.DLL.
  SQL_POSITION          = 0;      // possible 'Operation' parameters
  SQL_REFRESH           = 1;
  SQL_UPDATE            = 2;
  SQL_DELETE            = 3;

  SQL_LOCK_NO_CHANGE    = 0;      // possible 'LockType' parameters
  SQL_LOCK_EXCLUSIVE    = 1;
  SQL_LOCK_UNLOCK       = 2;

  // ODBC SQL Diag Variables
  SQL_DIAG_CURSOR_ROW_COUNT  = -1249;
  SQL_DIAG_ROW_NUMBER        = -1248;
  SQL_DIAG_COLUMN_NUMBER     = -1247;

  // ODBC Diag Values
  SQL_DIAG_RETURNCODE           = 1;
  SQL_DIAG_NUMBER               = 2;
  SQL_DIAG_ROW_COUNT            = 3;
  SQL_DIAG_SQLSTATE             = 4;
  SQL_DIAG_NATIVE               = 5;
  SQL_DIAG_MESSAGE_TEXT         = 6;
  SQL_DIAG_DYNAMIC_FUNCTION     = 7;
  SQL_DIAG_CLASS_ORIGIN         = 8;
  SQL_DIAG_SUBCLASS_ORIGIN      = 9;
  SQL_DIAG_CONNECTION_NAME      = 10;
  SQL_DIAG_SERVER_NAME          = 11;
  SQL_DIAG_DYNAMIC_FUNCTION_CODE = 12;

  // ODBC Connection Attributes (as used by SQLSetConnectAttr).
  SQL_ACCESS_MODE            = 101;
  SQL_AUTOCOMMIT             = 102;
  SQL_LOGIN_TIMEOUT          = 103;
  SQL_OPT_TRACE              = 104;
  SQL_OPT_TRACEFILE          = 105;
  SQL_TRANSLATE_DLL          = 106;
  SQL_TRANSLATE_OPTION       = 107;
  SQL_TXN_ISOLATION          = 108;
  SQL_ATTR_CURRENT_CATALOG   = 109;
  SQL_ODBC_CURSORS           = 110;
  SQL_QUIET_MODE             = 111;
  SQL_PACKET_SIZE            = 112;
  SQL_CONNECTION_TIMEOUT     = 113;
  SQL_DISCONNECT_BEHAVIOR    = 114;

  SQL_ATTR_LOGIN_TIMEOUT      = SQL_LOGIN_TIMEOUT;
  SQL_ATTR_CONNECTION_TIMEOUT = SQL_CONNECTION_TIMEOUT;

  SQL_ATTR_CONNECTION_DEAD    = 1209;   // GetConnectAttr only
    SQL_CD_TRUE               = 1;        // Connection is closed/dead
    SQL_CD_FALSE              = 0;       // Connection is open/available

  SQL_ENLIST_IN_DTC          = 1207;
  SQL_ENLIST_IN_XA           = 1208;
  SQL_ATTR_METADATA_ID       = 10014;
  
  // SQLColAttributes defines.
  SQL_COLUMN_COUNT           = 0;
  SQL_COLUMN_NAME            = 1;
  SQL_COLUMN_TYPE            = 2;
  SQL_COLUMN_LENGTH          = 3;
  SQL_COLUMN_PRECISION       = 4;
  SQL_COLUMN_SCALE           = 5;
  SQL_COLUMN_DISPLAY_SIZE    = 6;
  SQL_COLUMN_NULLABLE        = 7;
  SQL_COLUMN_UNSIGNED        = 8;
  SQL_COLUMN_MONEY           = 9;
  SQL_COLUMN_UPDATABLE       = 10;
  SQL_COLUMN_AUTO_INCREMENT  = 11;
  SQL_COLUMN_CASE_SENSITIVE  = 12;
  SQL_COLUMN_SEARCHABLE      = 13;
  SQL_COLUMN_TYPE_NAME       = 14;
  SQL_COLUMN_TABLE_NAME      = 15;
  SQL_COLUMN_OWNER_NAME      = 16;
  SQL_COLUMN_QUALIFIER_NAME  = 17;
  SQL_COLUMN_LABEL           = 18;
  SQL_COLUMN_DRIVER_START    = 1000;

  // Identifiers of fields in the SQL descriptor.  Used by SqlColAttributes() ...
  SQL_DESC_COUNT                  = 1001;
  SQL_DESC_TYPE                   = 1002;
  SQL_DESC_LENGTH                 = 1003;
  SQL_DESC_OCTET_LENGTH_PTR       = 1004;
  SQL_DESC_PRECISION              = 1005;
  SQL_DESC_SCALE                  = 1006;
  SQL_DESC_DATETIME_INTERVAL_CODE = 1007;
  SQL_DESC_NULLABLE               = 1008;
  SQL_DESC_INDICATOR_PTR          = 1009;
  SQL_DESC_DATA_PTR               = 1010;
  SQL_DESC_NAME                   = 1011;
  SQL_DESC_UNNAMED                = 1012;
  SQL_DESC_OCTET_LENGTH           = 1013;
  SQL_DESC_ALLOC_TYPE             = 1099;

  SQL_DESC_TYPE_NAME         = SQL_COLUMN_TYPE_NAME;
  SQL_DESC_DISPLAY_SIZE      = SQL_COLUMN_DISPLAY_SIZE;


  //
  SQL_INDEX_UNIQUE =   0;
  SQL_INDEX_ALL    =   1;

  SQL_QUICK        =    0;
  SQL_ENSURE       =    1;

  //========================================================
  // ODBC Statement Attributes (as used by SQLSetStmtAttr).
  //--------------------------------------------------------
  SQL_ATTR_AUTOCOMMIT        = SQL_AUTOCOMMIT;
  SQL_AUTOCOMMIT_OFF         = 0;
  SQL_AUTOCOMMIT_ON          = 1;
  SQL_AUTOCOMMIT_DEFAULT     = SQL_AUTOCOMMIT_ON;

  // SQLEndTran() options ...
  SQL_COMMIT                 = 0;
  SQL_ROLLBACK               = 1;

  // Used in SQLFreeStatement ...
  SQL_CLOSE                  = 0;
  SQL_DROP                   = 1;
  SQL_UNBIND                 = 2;
  SQL_RESET_PARAMS           = 3;

  SQL_ATTR_CURSOR_SCROLLABLE      = -1;
    // SQL_CURSOR_SCROLLABLE values
    SQL_NONSCROLLABLE = 0;
    SQL_SCROLLABLE    = 1;

  SQL_ATTR_CURSOR_SENSITIVITY     = -2;

  SQL_ATTR_ACCESS_MODE            = SQL_ACCESS_MODE;
    SQL_MODE_READ_WRITE           = 0;
    SQL_MODE_READ_ONLY            = 1;

  SQL_ATTR_ODBC_CURSORS       = SQL_ODBC_CURSORS;
    SQL_CUR_USE_IF_NEEDED    = 0;
    SQL_CUR_USE_ODBC         = 1;
    SQL_CUR_USE_DRIVER       = 2;
    SQL_CUR_DEFAULT          = SQL_CUR_USE_DRIVER;

  SQL_QUERY_TIMEOUT               = 0;
    // SQL_QUERY_TIMEOUT options
    SQL_QUERY_TIMEOUT_DEFAULT = 0;
  SQL_MAX_ROWS                    = 1;
    // SQL_MAX_ROWS options
    SQL_MAX_ROWS_DEFAULT = 0;

  SQL_ATTR_MAX_ROWS               = SQL_MAX_ROWS;

  SQL_NOSCAN                      = 2;
    // SQL_NOSCAN options
    SQL_NOSCAN_OFF = 0;           // is default value as well.
    SQL_NOSCAN_ON  = 1;
  SQL_MAX_LENGTH                  = 3;
    // SQL_MAX_LENGTH options
    SQL_MAX_LENGTH_DEFAULT = 0;

  SQL_ASYNC_ENABLE                = 4;
  SQL_ATTR_ASYNC_ENABLE           =   4;
    // values for SQL_ASYNC_ENABLE
    SQL_ASYNC_ENABLE_OFF = 0;     // is default value as well.
    SQL_ASYNC_ENABLE_ON  = 1;

  SQL_BIND_TYPE                   = 5;
    // SQL_BIND_TYPE options
    SQL_BIND_BY_COLUMN = 0;       // is default value as well.

  SQL_CURSOR_TYPE                 = 6;
  SQL_ATTR_CURSOR_TYPE            = SQL_CURSOR_TYPE;
    // SQL_CURSOR_TYPE options
    SQL_CURSOR_FORWARD_ONLY  = 0; // is default value as well.
    SQL_CURSOR_KEYSET_DRIVEN = 1;
    SQL_CURSOR_DYNAMIC       = 2;
    SQL_CURSOR_STATIC        = 3;

  SQL_CONCURRENCY                 = 7;
  SQL_ATTR_CONCURRENCY            = SQL_CONCURRENCY;
    // SQL_CONCURRENCY options
    SQL_CONCUR_READ_ONLY = 1;     // is default value as well.
    SQL_CONCUR_LOCK      = 2;
    SQL_CONCUR_ROWVER    = 3;
    SQL_CONCUR_VALUES    = 4;
  SQL_KEYSET_SIZE                 = 8;
    // SQL_KEYSET_SIZE options
    SQL_KEYSET_SIZE_DEFAULT = 0;
  SQL_ROWSET_SIZE                 = 9;
    // SQL_ROWSET_SIZE options
    SQL_ROWSET_SIZE_DEFAULT = 1;
  SQL_SIMULATE_CURSOR             = 10;
    // SQL_SIMULATE_CURSOR options
    SQL_SC_NON_UNIQUE = 0;
    SQL_SC_TRY_UNIQUE = 1;
    SQL_SC_UNIQUE     = 2;
  SQL_RETRIEVE_DATA               = 11;
    // SQL_RETRIEVE_DATA options
    SQL_RD_OFF = 0;
    SQL_RD_ON  = 1;               // is default value as well.
  SQL_USE_BOOKMARKS               = 12;
    // SQL_USE_BOOKMARKS options
    SQL_UB_OFF      = 0;          // is default value as well.
    SQL_UB_ON       = 1;
    SQL_UB_FIXED    = SQL_UB_ON;
    SQL_UB_VARIABLE = 2;
  SQL_GET_BOOKMARK                = 13;     // GetStmtOption Only!
  SQL_ROW_NUMBER                  = 14;     // GetStmtOption Only!
  SQL_ENABLE_AUTO_IPD             = 15;
  SQL_FETCH_BOOKMARK_PTR          = 16;
  SQL_PARAM_BIND_OFFSET_PTR       = 17;
  SQL_PARAM_BIND_TYPE             = 18;
    // the value of SQL_PARAM_BIND_TYPE
    SQL_PARAM_BIND_BY_COLUMN = 0; // is default value as well.
  SQL_PARAM_OPERATION_PTR         = 19;
  SQL_PARAM_STATUS_PTR            = 20;
  SQL_PARAMS_PROCESSED_PTR        = 21;
  SQL_PARAMSET_SIZE               = 22;
  SQL_ROW_BIND_OFFSET_PTR         = 23;
  SQL_ROW_OPERATION_PTR           = 24;
  SQL_ROW_STATUS_PTR              = 25;
  SQL_ROWS_FETCHED_PTR            = 26;
  SQL_ATTR_ROWS_FETCHED_PTR       = SQL_ROWS_FETCHED_PTR;

  SQL_ROW_ARRAY_SIZE              = 27;
  //========================================================

  // Used when getting the ROWID stuff ...
  SQL_BEST_ROWID                  = 1;
  SQL_ROWVER                      = 2;
  SQL_SCOPE_TRANSACTION           = 1;
  SQL_NO_NULLS                    = 0;
  SQL_NULLABLE                    = 1;

  //============================================================================
  // SQLGetInfo Information Types for ODBC (to analyse Environment, Connection,
  // and Statement level options).
  //----------------------------------------------------------------------------

  SQL_MAX_DRIVER_CONNECTIONS           = 0;
  SQL_MAX_CONCURRENT_ACTIVITIES        = 1;
  SQL_DATA_SOURCE_NAME                 = 2;
  SQL_DRIVER_HDBC                      = 3;
  SQL_DRIVER_HENV                      = 4;
  SQL_DRIVER_HSTMT                     = 5;
  SQL_DRIVER_NAME                      = 6;
  SQL_DRIVER_VER                       = 7;

  SQL_DRIVER_NOPROMPT                  = 0;
  SQL_DRIVER_COMPLETE                  = 1;
  SQL_DRIVER_PROMPT                    = 2;
  SQL_DRIVER_COMPLETE_REQUIRED         = 3;

  SQL_FETCH_DIRECTION                  = 8;
    // SQL_FETCH_DIRECTION bitmasks
    SQL_FD_FETCH_NEXT     = $00000001;
    SQL_FD_FETCH_FIRST    = $00000002;
    SQL_FD_FETCH_LAST     = $00000004;
    SQL_FD_FETCH_PRIOR    = $00000008;
    SQL_FD_FETCH_ABSOLUTE = $00000010;
    SQL_FD_FETCH_RELATIVE = $00000020;
    SQL_FD_FETCH_BOOKMARK = $00000080;
  SQL_ODBC_API_CONFORMANCE             = 9;
    // SQL_ODBC_API_CONFORMANCE values
    SQL_OAC_NONE   = $0000;
    SQL_OAC_LEVEL1 = $0001;
    SQL_OAC_LEVEL2 = $0002;
  SQL_ODBC_VER                         = 10;
  SQL_ROW_UPDATES                      = 11;
  SQL_ODBC_SAG_CLI_CONFORMANCE         = 12;
    // SQL_ODBC_SAG_CLI_CONFORMANCE values
    SQL_OSCC_NOT_COMPLIANT = $0000;
    SQL_OSCC_COMPLIANT     = $0001;
  SQL_SERVER_NAME                      = 13;
  SQL_SEARCH_PATTERN_ESCAPE            = 14;
  SQL_ODBC_SQL_CONFORMANCE             = 15;
    // SQL_ODBC_SQL_CONFORMANCE values
    SQL_OSC_MINIMUM  = $0000;
    SQL_OSC_CORE     = $0001;
    SQL_OSC_EXTENDED = $0002;
  SQL_DBMS_NAME                        = 17;
  SQL_DBMS_VER                         = 18;
  SQL_ACCESSIBLE_TABLES                = 19;
  SQL_ACCESSIBLE_PROCEDURES            = 20;
  SQL_PROCEDURES                       = 21;
  SQL_CONCAT_NULL_BEHAVIOR             = 22;
    // SQL_CONCAT_NULL_BEHAVIOR values
    SQL_CB_NULL     = $0000;
    SQL_CB_NON_NULL = $0001;
  SQL_CURSOR_COMMIT_BEHAVIOR           = 23;
    // SQL_CURSOR_COMMIT_BEHAVIOR values
    SQL_CB_DELETE   = 0;
    SQL_CB_CLOSE    = 1;
    SQL_CB_PRESERVE = 2;
  SQL_CURSOR_ROLLBACK_BEHAVIOR         = 24;
  SQL_DATA_SOURCE_READ_ONLY            = 25;
  SQL_DEFAULT_TXN_ISOLATION            = 26;
  SQL_EXPRESSIONS_IN_ORDERBY           = 27;
  SQL_IDENTIFIER_CASE                  = 28;
    // SQL_IDENTIFIER_CASE values
    SQL_IC_UPPER     = 1;
    SQL_IC_LOWER     = 2;
    SQL_IC_SENSITIVE = 3;
    SQL_IC_MIXED     = 4;
  SQL_IDENTIFIER_QUOTE_CHAR            = 29;
  SQL_MAX_COLUMN_NAME_LEN              = 30;
  SQL_MAX_CURSOR_NAME_LEN              = 31;
  SQL_MAX_SCHEMA_NAME_LEN              = 32;
  SQL_MAX_PROCEDURE_NAME_LEN           = 33;
  SQL_MAX_CATALOG_NAME_LEN             = 34;
  SQL_MAX_TABLE_NAME_LEN               = 35;
  SQL_MULT_RESULT_SETS                 = 36;
  SQL_MULTIPLE_ACTIVE_TXN              = 37;
  SQL_OUTER_JOINS                      = 38;
  SQL_SCHEMA_TERM                      = 39;
  SQL_PROCEDURE_TERM                   = 40;
  SQL_CATALOG_NAME_SEPARATOR           = 41;
  SQL_CATALOG_TERM                     = 42;
  SQL_SCROLL_CONCURRENCY               = 43;
    // SQL_SCROLL_CONCURRENCY bitmasks
    SQL_SCCO_READ_ONLY  = $00000001;
    SQL_SCCO_LOCK       = $00000002;
    SQL_SCCO_OPT_ROWVER = $00000004;
    SQL_SCCO_OPT_VALUES = $00000008;
  SQL_SCROLL_OPTIONS                   = 44;
    // SQL_SCROLL_OPTIONS masks
    SQL_SO_FORWARD_ONLY  = $00000001;
    SQL_SO_KEYSET_DRIVEN = $00000002;
    SQL_SO_DYNAMIC       = $00000004;
    SQL_SO_MIXED         = $00000008;
    SQL_SO_STATIC        = $00000010;
  SQL_TABLE_TERM                       = 45;
  SQL_TXN_CAPABLE                      = 46;
    // SQL_TXN_CAPABLE values
    SQL_TC_NONE       = 0;
    SQL_TC_DML        = 1;
    SQL_TC_ALL        = 2;
    SQL_TC_DDL_COMMIT = 3;
    SQL_TC_DDL_IGNORE = 4;
  SQL_USER_NAME                        = 47;
  SQL_CONVERT_FUNCTIONS                = 48;
    // SQL_CONVERT_FUNCTIONS functions
    SQL_FN_CVT_CONVERT = $00000001;
    SQL_FN_CVT_CAST    = $00000002;
  SQL_NUMERIC_FUNCTIONS                = 49;
    // SQL_NUMERIC_FUNCTIONS functions
    SQL_FN_NUM_ABS      = $00000001;
    SQL_FN_NUM_ACOS     = $00000002;
    SQL_FN_NUM_ASIN     = $00000004;
    SQL_FN_NUM_ATAN     = $00000008;
    SQL_FN_NUM_ATAN2    = $00000010;
    SQL_FN_NUM_CEILING  = $00000020;
    SQL_FN_NUM_COS      = $00000040;
    SQL_FN_NUM_COT      = $00000080;
    SQL_FN_NUM_EXP      = $00000100;
    SQL_FN_NUM_FLOOR    = $00000200;
    SQL_FN_NUM_LOG      = $00000400;
    SQL_FN_NUM_MOD      = $00000800;
    SQL_FN_NUM_SIGN     = $00001000;
    SQL_FN_NUM_SIN      = $00002000;
    SQL_FN_NUM_SQRT     = $00004000;
    SQL_FN_NUM_TAN      = $00008000;
    SQL_FN_NUM_PI       = $00010000;
    SQL_FN_NUM_RAND     = $00020000;
    SQL_FN_NUM_DEGREES  = $00040000;
    SQL_FN_NUM_LOG10    = $00080000;
    SQL_FN_NUM_POWER    = $00100000;
    SQL_FN_NUM_RADIANS  = $00200000;
    SQL_FN_NUM_ROUND    = $00400000;
    SQL_FN_NUM_TRUNCATE = $00800000;
  SQL_STRING_FUNCTIONS                 = 50;
    // SQL_STRING_FUNCTIONS functions
    SQL_FN_STR_CONCAT           = $00000001;
    SQL_FN_STR_INSERT           = $00000002;
    SQL_FN_STR_LEFT             = $00000004;
    SQL_FN_STR_LTRIM            = $00000008;
    SQL_FN_STR_LENGTH           = $00000010;
    SQL_FN_STR_LOCATE           = $00000020;
    SQL_FN_STR_LCASE            = $00000040;
    SQL_FN_STR_REPEAT           = $00000080;
    SQL_FN_STR_REPLACE          = $00000100;
    SQL_FN_STR_RIGHT            = $00000200;
    SQL_FN_STR_RTRIM            = $00000400;
    SQL_FN_STR_SUBSTRING        = $00000800;
    SQL_FN_STR_UCASE            = $00001000;
    SQL_FN_STR_ASCII            = $00002000;
    SQL_FN_STR_CHAR             = $00004000;
    SQL_FN_STR_DIFFERENCE       = $00008000;
    SQL_FN_STR_LOCATE_2         = $00010000;
    SQL_FN_STR_SOUNDEX          = $00020000;
    SQL_FN_STR_SPACE            = $00040000;
    SQL_FN_STR_BIT_LENGTH       = $00080000;
    SQL_FN_STR_CHAR_LENGTH      = $00100000;
    SQL_FN_STR_CHARACTER_LENGTH = $00200000;
    SQL_FN_STR_OCTET_LENGTH     = $00400000;
    SQL_FN_STR_POSITION         = $00800000;
  SQL_SYSTEM_FUNCTIONS                 = 51;
    // SQL_SYSTEM_FUNCTIONS functions
    SQL_FN_SYS_USERNAME = $00000001;
    SQL_FN_SYS_DBNAME   = $00000002;
    SQL_FN_SYS_IFNULL   = $00000004;
  SQL_TIMEDATE_FUNCTIONS               = 52;
    // SQL_TIMEDATE_FUNCTIONS functions
    SQL_FN_TD_NOW               = $00000001;
    SQL_FN_TD_CURDATE           = $00000002;
    SQL_FN_TD_DAYOFMONTH        = $00000004;
    SQL_FN_TD_DAYOFWEEK         = $00000008;
    SQL_FN_TD_DAYOFYEAR         = $00000010;
    SQL_FN_TD_MONTH             = $00000020;
    SQL_FN_TD_QUARTER           = $00000040;
    SQL_FN_TD_WEEK              = $00000080;
    SQL_FN_TD_YEAR              = $00000100;
    SQL_FN_TD_CURTIME           = $00000200;
    SQL_FN_TD_HOUR              = $00000400;
    SQL_FN_TD_MINUTE            = $00000800;
    SQL_FN_TD_SECOND            = $00001000;
    SQL_FN_TD_TIMESTAMPADD      = $00002000;
    SQL_FN_TD_TIMESTAMPDIFF     = $00004000;
    SQL_FN_TD_DAYNAME           = $00008000;
    SQL_FN_TD_MONTHNAME         = $00010000;
    SQL_FN_TD_CURRENT_DATE      = $00020000;
    SQL_FN_TD_CURRENT_TIME      = $00040000;
    SQL_FN_TD_CURRENT_TIMESTAMP = $00080000;
    SQL_FN_TD_EXTRACT           = $00100000;
  SQL_CONVERT_BIGINT                   = 53;
  SQL_CONVERT_BINARY                   = 54;
  SQL_CONVERT_BIT                      = 55;
  SQL_CONVERT_CHAR                     = 56;
  SQL_CONVERT_DATE                     = 57;
  SQL_CONVERT_DECIMAL                  = 58;
  SQL_CONVERT_DOUBLE                   = 59;
  SQL_CONVERT_FLOAT                    = 60;
  SQL_CONVERT_INTEGER                  = 61;
  SQL_CONVERT_LONGVARCHAR              = 62;
  SQL_CONVERT_NUMERIC                  = 63;
  SQL_CONVERT_REAL                     = 64;
  SQL_CONVERT_SMALLINT                 = 65;
  SQL_CONVERT_TIME                     = 66;
  SQL_CONVERT_TIMESTAMP                = 67;
  SQL_CONVERT_TINYINT                  = 68;
  SQL_CONVERT_VARBINARY                = 69;
  SQL_CONVERT_VARCHAR                  = 70;
  SQL_CONVERT_LONGVARBINARY            = 71;
    // SQL_CONVERT_*  return value bitmasks
    SQL_CVT_CHAR                = $00000001;
    SQL_CVT_NUMERIC             = $00000002;
    SQL_CVT_DECIMAL             = $00000004;
    SQL_CVT_INTEGER             = $00000008;
    SQL_CVT_SMALLINT            = $00000010;
    SQL_CVT_FLOAT               = $00000020;
    SQL_CVT_REAL                = $00000040;
    SQL_CVT_DOUBLE              = $00000080;
    SQL_CVT_VARCHAR             = $00000100;
    SQL_CVT_LONGVARCHAR         = $00000200;
    SQL_CVT_BINARY              = $00000400;
    SQL_CVT_VARBINARY           = $00000800;
    SQL_CVT_BIT                 = $00001000;
    SQL_CVT_TINYINT             = $00002000;
    SQL_CVT_BIGINT              = $00004000;
    SQL_CVT_DATE                = $00008000;
    SQL_CVT_TIME                = $00010000;
    SQL_CVT_TIMESTAMP           = $00020000;
    SQL_CVT_LONGVARBINARY       = $00040000;
    SQL_CVT_INTERVAL_YEAR_MONTH = $00080000;
    SQL_CVT_INTERVAL_DAY_TIME   = $00100000;
    SQL_CVT_WCHAR               = $00200000;
    SQL_CVT_WLONGVARCHAR        = $00400000;
    SQL_CVT_WVARCHAR            = $00800000;
  SQL_TXN_ISOLATION_OPTION             = 72;
    // SQL_TXN_ISOLATION_OPTION bitmasks
    SQL_TXN_READ_UNCOMMITTED = $00000001;
    SQL_TXN_READ_COMMITTED   = $00000002;
    SQL_TXN_REPEATABLE_READ  = $00000004;
    SQL_TXN_SERIALIZABLE     = $00000008;
  SQL_INTEGRITY                        = 73;
  SQL_CORRELATION_NAME                 = 74;
    // SQL_CORRELATION_NAME values
    SQL_CN_NONE      = $0000;
    SQL_CN_DIFFERENT = $0001;
    SQL_CN_ANY       = $0002;
  SQL_NON_NULLABLE_COLUMNS             = 75;
    // SQL_NON_NULLABLE_COLUMNS values
    SQL_NNC_NULL     = $0000;
    SQL_NNC_NON_NULL = $0001;
  SQL_DRIVER_HLIB                      = 76;
  SQL_DRIVER_ODBC_VER                  = 77;
  SQL_LOCK_TYPES                       = 78;
    // SQL_LOCK_TYPES masks
    SQL_LCK_NO_CHANGE = $00000001;
    SQL_LCK_EXCLUSIVE = $00000002;
    SQL_LCK_UNLOCK    = $00000004;
  SQL_POS_OPERATIONS                   = 79;
    // SQL_POS_OPERATIONS masks
    SQL_POS_POSITION = $00000001;
    SQL_POS_REFRESH  = $00000002;
    SQL_POS_UPDATE   = $00000004;
    SQL_POS_DELETE   = $00000008;
    SQL_POS_ADD      = $00000010;
  SQL_POSITIONED_STATEMENTS            = 80;
    // SQL_POSITIONED_STATEMENTS masks
    SQL_PS_POSITIONED_DELETE = $00000001;
    SQL_PS_POSITIONED_UPDATE = $00000002;
    SQL_PS_SELECT_FOR_UPDATE = $00000004;
  SQL_GETDATA_EXTENSIONS               = 81;
    // SQL_GETDATA_EXTENSIONS bitmasks
    SQL_GD_ANY_COLUMN = $00000001;
    SQL_GD_ANY_ORDER  = $00000002;
    SQL_GD_BLOCK      = $00000004;
    SQL_GD_BOUND      = $00000008;
  SQL_BOOKMARK_PERSISTENCE             = 82;
    // SQL_BOOKMARK_PERSISTENCE values
    SQL_BP_CLOSE       = $00000001;
    SQL_BP_DELETE      = $00000002;
    SQL_BP_DROP        = $00000004;
    SQL_BP_TRANSACTION = $00000008;
    SQL_BP_UPDATE      = $00000010;
    SQL_BP_OTHER_HSTMT = $00000020;
    SQL_BP_SCROLL      = $00000040;
  SQL_STATIC_SENSITIVITY               = 83;
    // SQL_STATIC_SENSITIVITY values
    SQL_SS_ADDITIONS = $00000001;
    SQL_SS_DELETIONS = $00000002;
    SQL_SS_UPDATES   = $00000004;
  SQL_FILE_USAGE                       = 84;
    // SQL_FILE_USAGE values
    SQL_FILE_NOT_SUPPORTED = $0000;
    SQL_FILE_TABLE         = $0001;
    SQL_FILE_CATALOG       = $0002;
  SQL_NULL_COLLATION                   = 85;
    // SQL_NULL_COLLATION values
    SQL_NC_HIGH  = 0;
    SQL_NC_LOW   = 1;
    SQL_NC_START = $0002;
    SQL_NC_END   = $0004;
  SQL_ALTER_TABLE                      = 86;
    // SQL_ALTER_TABLE bitmasks
    SQL_AT_ADD_COLUMN                     = $00000001;
    SQL_AT_DROP_COLUMN                    = $00000002;
    SQL_AT_ADD_CONSTRAINT                 = $00000008;
    SQL_AT_ADD_COLUMN_SINGLE              = $00000020;
    SQL_AT_ADD_COLUMN_DEFAULT             = $00000040;
    SQL_AT_ADD_COLUMN_COLLATION           = $00000080;
    SQL_AT_SET_COLUMN_DEFAULT             = $00000100;
    SQL_AT_DROP_COLUMN_DEFAULT            = $00000200;
    SQL_AT_DROP_COLUMN_CASCADE            = $00000400;
    SQL_AT_DROP_COLUMN_RESTRICT           = $00000800;
    SQL_AT_ADD_TABLE_CONSTRAINT           = $00001000;
    SQL_AT_DROP_TABLE_CONSTRAINT_CASCADE  = $00002000;
    SQL_AT_DROP_TABLE_CONSTRAINT_RESTRICT = $00004000;
    SQL_AT_CONSTRAINT_NAME_DEFINITION     = $00008000;
    SQL_AT_CONSTRAINT_INITIALLY_DEFERRED  = $00010000;
    SQL_AT_CONSTRAINT_INITIALLY_IMMEDIATE = $00020000;
    SQL_AT_CONSTRAINT_DEFERRABLE          = $00040000;
    SQL_AT_CONSTRAINT_NON_DEFERRABLE      = $00080000;
  SQL_COLUMN_ALIAS                     = 87;
  SQL_GROUP_BY                         = 88;
    // SQL_GROUP_BY values
    SQL_GB_NOT_SUPPORTED            = $0000;
    SQL_GB_GROUP_BY_EQUALS_SELECT   = $0001;
    SQL_GB_GROUP_BY_CONTAINS_SELECT = $0002;
    SQL_GB_NO_RELATION              = $0003;
    SQL_GB_COLLATE                  = $0004;
  SQL_KEYWORDS                         = 89;
  SQL_ORDER_BY_COLUMNS_IN_SELECT       = 90;
  SQL_SCHEMA_USAGE                     = 91;
    // SQL_SCHEMA_USAGE masks
    SQL_SU_DML_STATEMENTS       = $00000001;
    SQL_SU_PROCEDURE_INVOCATION = $00000002;
    SQL_SU_TABLE_DEFINITION     = $00000004;
    SQL_SU_INDEX_DEFINITION     = $00000008;
    SQL_SU_PRIVILEGE_DEFINITION = $00000010;
  SQL_CATALOG_USAGE                    = 92;
    // SQL_CATALOG_USAGE masks
    SQL_CU_DML_STATEMENTS       = $00000001;
    SQL_CU_PROCEDURE_INVOCATION = $00000002;
    SQL_CU_TABLE_DEFINITION     = $00000004;
    SQL_CU_INDEX_DEFINITION     = $00000008;
    SQL_CU_PRIVILEGE_DEFINITION = $00000010;
  SQL_QUOTED_IDENTIFIER_CASE           = 93;
  SQL_SPECIAL_CHARACTERS               = 94;
  SQL_SUBQUERIES                       = 95;
    // SQL_SUBQUERIES masks
    SQL_SQ_COMPARISON            = $00000001;
    SQL_SQ_EXISTS                = $00000002;
    SQL_SQ_IN                    = $00000004;
    SQL_SQ_QUANTIFIED            = $00000008;
    SQL_SQ_CORRELATED_SUBQUERIES = $00000010;
  SQL_UNION                            = 96;
    // SQL_UNION masks
    SQL_U_UNION     = $00000001;
    SQL_U_UNION_ALL = $00000002;
  SQL_MAX_COLUMNS_IN_GROUP_BY          = 97;
  SQL_MAX_COLUMNS_IN_INDEX             = 98;
  SQL_MAX_COLUMNS_IN_ORDER_BY          = 99;
  SQL_MAX_COLUMNS_IN_SELECT            = 100;
  SQL_MAX_COLUMNS_IN_TABLE             = 101;
  SQL_MAX_INDEX_SIZE                   = 102;
  SQL_MAX_ROW_SIZE_INCLUDES_LONG       = 103;
  SQL_MAX_ROW_SIZE                     = 104;
  SQL_MAX_STATEMENT_LEN                = 105;
  SQL_MAX_TABLES_IN_SELECT             = 106;
  SQL_MAX_USER_NAME_LEN                = 107;
  SQL_MAX_CHAR_LITERAL_LEN             = 108;
  SQL_TIMEDATE_ADD_INTERVALS           = 109;
  SQL_TIMEDATE_DIFF_INTERVALS          = 110;
    // SQL_TIMEDATE_ADD_INTERVALS and SQL_TIMEDATE_DIFF_INTERVALS functions
    SQL_FN_TSI_FRAC_SECOND = $00000001;
    SQL_FN_TSI_SECOND      = $00000002;
    SQL_FN_TSI_MINUTE      = $00000004;
    SQL_FN_TSI_HOUR        = $00000008;
    SQL_FN_TSI_DAY         = $00000010;
    SQL_FN_TSI_WEEK        = $00000020;
    SQL_FN_TSI_MONTH       = $00000040;
    SQL_FN_TSI_QUARTER     = $00000080;
    SQL_FN_TSI_YEAR        = $00000100;
  SQL_NEED_LONG_DATA_LEN               = 111;
  SQL_MAX_BINARY_LITERAL_LEN           = 112;
  SQL_LIKE_ESCAPE_CLAUSE               = 113;
  SQL_CATALOG_LOCATION                 = 114;
    // SQL_CATALOG_LOCATION values
    SQL_CL_START = $0001;
    SQL_CL_END   = $0002;
  SQL_OJ_CAPABILITIES                  = 115;
    // SQL_OJ_CAPABILITIES bitmasks (NB: this means "outer join"', not what you may be thinking)
    SQL_OJ_LEFT               = $00000001;
    SQL_OJ_RIGHT              = $00000002;
    SQL_OJ_FULL               = $00000004;
    SQL_OJ_NESTED             = $00000008;
    SQL_OJ_NOT_ORDERED        = $00000010;
    SQL_OJ_INNER              = $00000020;
    SQL_OJ_ALL_COMPARISON_OPS = $00000040;
  SQL_ACTIVE_ENVIRONMENTS              = 116;
  SQL_ALTER_DOMAIN                     = 117;
    // Bitmasks for SQL_ALTER_DOMAIN
    SQL_AD_CONSTRAINT_NAME_DEFINITION         = $00000001;
    SQL_AD_ADD_DOMAIN_CONSTRAINT              = $00000002;
    SQL_AD_DROP_DOMAIN_CONSTRAINT             = $00000004;
    SQL_AD_ADD_DOMAIN_DEFAULT                 = $00000008;
    SQL_AD_DROP_DOMAIN_DEFAULT                = $00000010;
    SQL_AD_ADD_CONSTRAINT_INITIALLY_DEFERRED  = $00000020;
    SQL_AD_ADD_CONSTRAINT_INITIALLY_IMMEDIATE = $00000040;
    SQL_AD_ADD_CONSTRAINT_DEFERRABLE          = $00000080;
    SQL_AD_ADD_CONSTRAINT_NON_DEFERRABLE      = $00000100;
  SQL_SQL_CONFORMANCE                  = 118;
    // SQL_SQL_CONFORMANCE bit masks
    SQL_SC_SQL92_ENTRY            = $00000001;
    SQL_SC_FIPS127_2_TRANSITIONAL = $00000002;
    SQL_SC_SQL92_INTERMEDIATE     = $00000004;
    SQL_SC_SQL92_FULL             = $00000008;
  SQL_DATETIME_LITERALS                = 119;
    // SQL_DATETIME_LITERALS masks
    SQL_DL_SQL92_DATE                      = $00000001;
    SQL_DL_SQL92_TIME                      = $00000002;
    SQL_DL_SQL92_TIMESTAMP                 = $00000004;
    SQL_DL_SQL92_INTERVAL_YEAR             = $00000008;
    SQL_DL_SQL92_INTERVAL_MONTH            = $00000010;
    SQL_DL_SQL92_INTERVAL_DAY              = $00000020;
    SQL_DL_SQL92_INTERVAL_HOUR             = $00000040;
    SQL_DL_SQL92_INTERVAL_MINUTE           = $00000080;
    SQL_DL_SQL92_INTERVAL_SECOND           = $00000100;
    SQL_DL_SQL92_INTERVAL_YEAR_TO_MONTH    = $00000200;
    SQL_DL_SQL92_INTERVAL_DAY_TO_HOUR      = $00000400;
    SQL_DL_SQL92_INTERVAL_DAY_TO_MINUTE    = $00000800;
    SQL_DL_SQL92_INTERVAL_DAY_TO_SECOND    = $00001000;
    SQL_DL_SQL92_INTERVAL_HOUR_TO_MINUTE   = $00002000;
    SQL_DL_SQL92_INTERVAL_HOUR_TO_SECOND   = $00004000;
    SQL_DL_SQL92_INTERVAL_MINUTE_TO_SECOND = $00008000;
  SQL_BATCH_ROW_COUNT                  = 120;
    // values for SQL_BATCH_ROW_COUNT
    SQL_BRC_PROCEDURES = $0000001;
    SQL_BRC_EXPLICIT   = $0000002;
    SQL_BRC_ROLLED_UP  = $0000004;
  SQL_BATCH_SUPPORT                    = 121;
    // bitmasks for SQL_BATCH_SUPPORT
    SQL_BS_SELECT_EXPLICIT    = $00000001;
    SQL_BS_ROW_COUNT_EXPLICIT = $00000002;
    SQL_BS_SELECT_PROC        = $00000004;
    SQL_BS_ROW_COUNT_PROC     = $00000008;
  SQL_CONVERT_WCHAR                    = 122;
  SQL_CONVERT_INTERVAL_DAY_TIME        = 123;
  SQL_CONVERT_INTERVAL_YEAR_MONTH      = 124;
  SQL_CONVERT_WLONGVARCHAR             = 125;
  SQL_CONVERT_WVARCHAR                 = 126;
  SQL_CREATE_ASSERTION                 = 127;
    // SQL_CREATE_ASSERTION bitmasks
    SQL_CA_CREATE_ASSERTION               = $00000001;
    SQL_CA_CONSTRAINT_INITIALLY_DEFERRED  = $00000010;
    SQL_CA_CONSTRAINT_INITIALLY_IMMEDIATE = $00000020;
    SQL_CA_CONSTRAINT_DEFERRABLE          = $00000040;
    SQL_CA_CONSTRAINT_NON_DEFERRABLE      = $00000080;
  SQL_CREATE_CHARACTER_SET             = 128;
    // SQL_CREATE_CHARACTER_SET bitmasks
    SQL_CCS_CREATE_CHARACTER_SET = $00000001;
    SQL_CCS_COLLATE_CLAUSE       = $00000002;
    SQL_CCS_LIMITED_COLLATION    = $00000004;
  SQL_CREATE_COLLATION                 = 129;
    // SQL_CREATE_COLLATION bitmasks
    SQL_CCOL_CREATE_COLLATION = $00000001;
  SQL_CREATE_DOMAIN                    = 130;
    // SQL_CREATE_DOMAIN bitmasks
    SQL_CDO_CREATE_DOMAIN                  = $00000001;
    SQL_CDO_DEFAULT                        = $00000002;
    SQL_CDO_CONSTRAINT                     = $00000004;
    SQL_CDO_COLLATION                      = $00000008;
    SQL_CDO_CONSTRAINT_NAME_DEFINITION     = $00000010;
    SQL_CDO_CONSTRAINT_INITIALLY_DEFERRED  = $00000020;
    SQL_CDO_CONSTRAINT_INITIALLY_IMMEDIATE = $00000040;
    SQL_CDO_CONSTRAINT_DEFERRABLE          = $00000080;
    SQL_CDO_CONSTRAINT_NON_DEFERRABLE      = $00000100;
  SQL_CREATE_SCHEMA                    = 131;
    // SQL_CREATE_SCHEMA bitmasks
    SQL_CS_CREATE_SCHEMA         = $00000001;
    SQL_CS_AUTHORIZATION         = $00000002;
    SQL_CS_DEFAULT_CHARACTER_SET = $00000004;
  SQL_CREATE_TABLE                     = 132;
    // SQL_CREATE_TABLE bitmasks
    SQL_CT_CREATE_TABLE                   = $00000001;
    SQL_CT_COMMIT_PRESERVE                = $00000002;
    SQL_CT_COMMIT_DELETE                  = $00000004;
    SQL_CT_GLOBAL_TEMPORARY               = $00000008;
    SQL_CT_LOCAL_TEMPORARY                = $00000010;
    SQL_CT_CONSTRAINT_INITIALLY_DEFERRED  = $00000020;
    SQL_CT_CONSTRAINT_INITIALLY_IMMEDIATE = $00000040;
    SQL_CT_CONSTRAINT_DEFERRABLE          = $00000080;
    SQL_CT_CONSTRAINT_NON_DEFERRABLE      = $00000100;
    SQL_CT_COLUMN_CONSTRAINT              = $00000200;
    SQL_CT_COLUMN_DEFAULT                 = $00000400;
    SQL_CT_COLUMN_COLLATION               = $00000800;
    SQL_CT_TABLE_CONSTRAINT               = $00001000;
    SQL_CT_CONSTRAINT_NAME_DEFINITION     = $00002000;
  SQL_CREATE_TRANSLATION               = 133;
    // SQL_CREATE_TRANSLATION bitmasks
    SQL_CTR_CREATE_TRANSLATION = $00000001;
  SQL_CREATE_VIEW                      = 134;
    // SQL_CREATE_VIEW values
    SQL_CV_CREATE_VIEW  = $00000001;
    SQL_CV_CHECK_OPTION = $00000002;
    SQL_CV_CASCADED     = $00000004;
    SQL_CV_LOCAL        = $00000008;
  SQL_DRIVER_HDESC                     = 135;
  SQL_DROP_ASSERTION                   = 136;
    // SQL_DROP_ASSERTION bitmasks
    SQL_DA_DROP_ASSERTION = $00000001;
  SQL_DROP_CHARACTER_SET               = 137;
    // SQL_DROP_CHARACTER_SET bitmasks
    SQL_DCS_DROP_CHARACTER_SET = $00000001;
  SQL_DROP_COLLATION                   = 138;
    // SQL_DROP_COLLATION bitmasks
    SQL_DC_DROP_COLLATION = $00000001;
  SQL_DROP_DOMAIN                      = 139;
    // SQL_DROP_DOMAIN bitmasks
    SQL_DD_DROP_DOMAIN = $00000001;
    SQL_DD_RESTRICT    = $00000002;
    SQL_DD_CASCADE     = $00000004;
  SQL_DROP_SCHEMA                      = 140;
    // SQL_DROP_SCHEMA bitmasks
    SQL_DS_DROP_SCHEMA = $00000001;
    SQL_DS_RESTRICT    = $00000002;
    SQL_DS_CASCADE     = $00000004;
  SQL_DROP_TABLE                       = 141;
    // SQL_DROP_TABLE bitmasks
    SQL_DT_DROP_TABLE = $00000001;
    SQL_DT_RESTRICT   = $00000002;
    SQL_DT_CASCADE    = $00000004;
  SQL_DROP_TRANSLATION                 = 142;
    // SQL_DROP_TRANSLATION bitmasks
    SQL_DTR_DROP_TRANSLATION = $00000001;
  SQL_DROP_VIEW                        = 143;
    // SQL_DROP_VIEW bitmasks
    SQL_DV_DROP_VIEW = $00000001;
    SQL_DV_RESTRICT  = $00000002;
    SQL_DV_CASCADE    = $00000004;
  SQL_DYNAMIC_CURSOR_ATTRIBUTES1       = 144;
    { bitmasks for SQL_DYNAMIC_CURSOR_ATTRIBUTES1, SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES1,
      SQL_KEYSET_CURSOR_ATTRIBUTES1, and SQL_STATIC_CURSOR_ATTRIBUTES1 }

    // supported SQLFetchScroll FetchOrientation's
    SQL_CA1_NEXT                    = $00000001;
    SQL_CA1_ABSOLUTE                = $00000002;
    SQL_CA1_RELATIVE                = $00000004;
    SQL_CA1_BOOKMARK                = $00000008;
    // supported SQLSetPos LockType's
    SQL_CA1_LOCK_NO_CHANGE          = $00000040;
    SQL_CA1_LOCK_EXCLUSIVE          = $00000080;
    SQL_CA1_LOCK_UNLOCK             = $00000100;
    // supported SQLSetPos Operations
    SQL_CA1_POS_POSITION            = $00000200;
    SQL_CA1_POS_UPDATE              = $00000400;
    SQL_CA1_POS_DELETE              = $00000800;
    SQL_CA1_POS_REFRESH             = $00001000;
    // positioned updates and deletes
    SQL_CA1_POSITIONED_UPDATE       = $00002000;
    SQL_CA1_POSITIONED_DELETE       = $00004000;
    SQL_CA1_SELECT_FOR_UPDATE       = $00008000;
    // supported SQLBulkOperations operations
    SQL_CA1_BULK_ADD                = $00010000;
    SQL_CA1_BULK_UPDATE_BY_BOOKMARK = $00020000;
    SQL_CA1_BULK_DELETE_BY_BOOKMARK = $00040000;
    SQL_CA1_BULK_FETCH_BY_BOOKMARK  = $00080000;
  SQL_DYNAMIC_CURSOR_ATTRIBUTES2       = 145;
    { bitmasks for SQL_DYNAMIC_CURSOR_ATTRIBUTES2, SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES2,
      SQL_KEYSET_CURSOR_ATTRIBUTES2, and SQL_STATIC_CURSOR_ATTRIBUTES2 }

    // supported values for SQL_ATTR_SCROLL_CONCURRENCY
    SQL_CA2_READ_ONLY_CONCURRENCY  = $00000001;
    SQL_CA2_LOCK_CONCURRENCY       = $00000002;
    SQL_CA2_OPT_ROWVER_CONCURRENCY = $00000004;
    SQL_CA2_OPT_VALUES_CONCURRENCY = $00000008;
    // sensitivity of the cursor to its own inserts, deletes, and updates
    SQL_CA2_SENSITIVITY_ADDITIONS  = $00000010;
    SQL_CA2_SENSITIVITY_DELETIONS  = $00000020;
    SQL_CA2_SENSITIVITY_UPDATES    = $00000040;
    // semantics of SQL_ATTR_MAX_ROWS
    SQL_CA2_MAX_ROWS_SELECT        = $00000080;
    SQL_CA2_MAX_ROWS_INSERT        = $00000100;
    SQL_CA2_MAX_ROWS_DELETE        = $00000200;
    SQL_CA2_MAX_ROWS_UPDATE        = $00000400;
    SQL_CA2_MAX_ROWS_CATALOG       = $00000800;
    SQL_CA2_MAX_ROWS_AFFECTS_ALL   = (SQL_CA2_MAX_ROWS_SELECT or SQL_CA2_MAX_ROWS_INSERT
                                      or SQL_CA2_MAX_ROWS_DELETE or SQL_CA2_MAX_ROWS_UPDATE
                                      or SQL_CA2_MAX_ROWS_CATALOG);
    // semantics of SQL_DIAG_CURSOR_ROW_COUNT
    SQL_CA2_CRC_EXACT              = $00001000;
    SQL_CA2_CRC_APPROXIMATE        = $00002000;
    // the kinds of positioned statements that can be simulated
    SQL_CA2_SIMULATE_NON_UNIQUE    = $00004000;
    SQL_CA2_SIMULATE_TRY_UNIQUE    = $00008000;
    SQL_CA2_SIMULATE_UNIQUE        = $00010000;
  SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES1  = 146;
  SQL_FORWARD_ONLY_CURSOR_ATTRIBUTES2  = 147;
  SQL_INDEX_KEYWORDS                   = 148;
    // Bitmasks for SQL_INDEX_KEYWORDS
    SQL_IK_NONE = $00000000;
    SQL_IK_ASC  = $00000001;
    SQL_IK_DESC = $00000002;
    SQL_IK_ALL  = (SQL_IK_ASC or SQL_IK_DESC);
  SQL_INFO_SCHEMA_VIEWS                = 149;
    // Bitmasks for SQL_INFO_SCHEMA_VIEWS
    SQL_ISV_ASSERTIONS              = $00000001;
    SQL_ISV_CHARACTER_SETS          = $00000002;
    SQL_ISV_CHECK_CONSTRAINTS       = $00000004;
    SQL_ISV_COLLATIONS              = $00000008;
    SQL_ISV_COLUMN_DOMAIN_USAGE     = $00000010;
    SQL_ISV_COLUMN_PRIVILEGES       = $00000020;
    SQL_ISV_COLUMNS                 = $00000040;
    SQL_ISV_CONSTRAINT_COLUMN_USAGE = $00000080;
    SQL_ISV_CONSTRAINT_TABLE_USAGE  = $00000100;
    SQL_ISV_DOMAIN_CONSTRAINTS      = $00000200;
    SQL_ISV_DOMAINS                 = $00000400;
    SQL_ISV_KEY_COLUMN_USAGE        = $00000800;
    SQL_ISV_REFERENTIAL_CONSTRAINTS = $00001000;
    SQL_ISV_SCHEMATA                = $00002000;
    SQL_ISV_SQL_LANGUAGES           = $00004000;
    SQL_ISV_TABLE_CONSTRAINTS       = $00008000;
    SQL_ISV_TABLE_PRIVILEGES        = $00010000;
    SQL_ISV_TABLES                  = $00020000;
    SQL_ISV_TRANSLATIONS            = $00040000;
    SQL_ISV_USAGE_PRIVILEGES        = $00080000;
    SQL_ISV_VIEW_COLUMN_USAGE       = $00100000;
    SQL_ISV_VIEW_TABLE_USAGE        = $00200000;
    SQL_ISV_VIEWS                   = $00400000;
  SQL_KEYSET_CURSOR_ATTRIBUTES1        = 150;
  SQL_KEYSET_CURSOR_ATTRIBUTES2        = 151;
  SQL_ODBC_INTERFACE_CONFORMANCE       = 152;
    // SQL_ODBC_INTERFACE_CONFORMANCE values
    SQL_OIC_CORE   = 1;
    SQL_OIC_LEVEL1 = 2;
    SQL_OIC_LEVEL2 = 3;
  SQL_PARAM_ARRAY_ROW_COUNTS           = 153;
    // Values for SQL_PARAM_ARRAY_ROW_COUNTS getinfo
    SQL_PARC_BATCH    = 1;
    SQL_PARC_NO_BATCH = 2;
  SQL_PARAM_ARRAY_SELECTS              = 154;
    // values for SQL_PARAM_ARRAY_SELECT_BATCH
    SQL_PAS_BATCH     = 1;
    SQL_PAS_NO_BATCH  = 2;
    SQL_PAS_NO_SELECT = 3;
  SQL_SQL92_DATETIME_FUNCTIONS         = 155;
    // SQL_SQL92_DATETIME_FUNCTIONS
    SQL_SDF_CURRENT_DATE      = 00000001;
    SQL_SDF_CURRENT_TIME      = 00000002;
    SQL_SDF_CURRENT_TIMESTAMP = 00000004;
  SQL_SQL92_FOREIGN_KEY_DELETE_RULE    = 156;
    // SQL_SQL92_FOREIGN_KEY_DELETE_RULE bitmasks
    SQL_SFKD_CASCADE     = $00000001;
    SQL_SFKD_NO_ACTION   = $00000002;
    SQL_SFKD_SET_DEFAULT = $00000004;
    SQL_SFKD_SET_NULL    = $00000008;
  SQL_SQL92_FOREIGN_KEY_UPDATE_RULE    = 157;
    // SQL_SQL92_FOREIGN_KEY_UPDATE_RULE bitmasks
    SQL_SFKU_CASCADE     = $00000001;
    SQL_SFKU_NO_ACTION   = $00000002;
    SQL_SFKU_SET_DEFAULT = $00000004;
    SQL_SFKU_SET_NULL    = $00000008;
  SQL_SQL92_GRANT                      = 158;
    // SQL_SQL92_GRANT  bitmasks
    SQL_SG_USAGE_ON_DOMAIN        = $00000001;
    SQL_SG_USAGE_ON_CHARACTER_SET = $00000002;
    SQL_SG_USAGE_ON_COLLATION     = $00000004;
    SQL_SG_USAGE_ON_TRANSLATION   = $00000008;
    SQL_SG_WITH_GRANT_OPTION      = $00000010;
    SQL_SG_DELETE_TABLE           = $00000020;
    SQL_SG_INSERT_TABLE           = $00000040;
    SQL_SG_INSERT_COLUMN          = $00000080;
    SQL_SG_REFERENCES_TABLE       = $00000100;
    SQL_SG_REFERENCES_COLUMN      = $00000200;
    SQL_SG_SELECT_TABLE           = $00000400;
    SQL_SG_UPDATE_TABLE           = $00000800;
    SQL_SG_UPDATE_COLUMN          = $00001000;
  SQL_SQL92_NUMERIC_VALUE_FUNCTIONS    = 159;
    // SQL_SQL92_NUMERIC_VALUE_FUNCTIONS
    SQL_SNVF_BIT_LENGTH       = $00000001;
    SQL_SNVF_CHAR_LENGTH      = $00000002;
    SQL_SNVF_CHARACTER_LENGTH = $00000004;
    SQL_SNVF_EXTRACT          = $00000008;
    SQL_SNVF_OCTET_LENGTH     = $00000010;
    SQL_SNVF_POSITION         = $00000020;
  SQL_SQL92_PREDICATES                 = 160;
    // SQL_SQL92_PREDICATES bitmasks
    SQL_SP_EXISTS                = $00000001;
    SQL_SP_ISNOTNULL             = $00000002;
    SQL_SP_ISNULL                = $00000004;
    SQL_SP_MATCH_FULL            = $00000008;
    SQL_SP_MATCH_PARTIAL         = $00000010;
    SQL_SP_MATCH_UNIQUE_FULL     = $00000020;
    SQL_SP_MATCH_UNIQUE_PARTIAL  = $00000040;
    SQL_SP_OVERLAPS              = $00000080;
    SQL_SP_UNIQUE                = $00000100;
    SQL_SP_LIKE                  = $00000200;
    SQL_SP_IN                    = $00000400;
    SQL_SP_BETWEEN               = $00000800;
    SQL_SP_COMPARISON            = $00001000;
    SQL_SP_QUANTIFIED_COMPARISON = $00002000;
  SQL_SQL92_RELATIONAL_JOIN_OPERATORS  = 161;
    // SQL_SQL92_RELATIONAL_JOIN_OPERATORS bitmasks
    SQL_SRJO_CORRESPONDING_CLAUSE = $00000001;
    SQL_SRJO_CROSS_JOIN           = $00000002;
    SQL_SRJO_EXCEPT_JOIN          = $00000004;
    SQL_SRJO_FULL_OUTER_JOIN      = $00000008;
    SQL_SRJO_INNER_JOIN           = $00000010;
    SQL_SRJO_INTERSECT_JOIN       = $00000020;
    SQL_SRJO_LEFT_OUTER_JOIN      = $00000040;
    SQL_SRJO_NATURAL_JOIN         = $00000080;
    SQL_SRJO_RIGHT_OUTER_JOIN     = $00000100;
    SQL_SRJO_UNION_JOIN           = $00000200;
  SQL_SQL92_REVOKE                     = 162;
    // SQL_SQL92_REVOKE bitmasks
    SQL_SR_USAGE_ON_DOMAIN        = $00000001;
    SQL_SR_USAGE_ON_CHARACTER_SET = $00000002;
    SQL_SR_USAGE_ON_COLLATION     = $00000004;
    SQL_SR_USAGE_ON_TRANSLATION   = $00000008;
    SQL_SR_GRANT_OPTION_FOR       = $00000010;
    SQL_SR_CASCADE                = $00000020;
    SQL_SR_RESTRICT               = $00000040;
    SQL_SR_DELETE_TABLE           = $00000080;
    SQL_SR_INSERT_TABLE           = $00000100;
    SQL_SR_INSERT_COLUMN          = $00000200;
    SQL_SR_REFERENCES_TABLE       = $00000400;
    SQL_SR_REFERENCES_COLUMN      = $00000800;
    SQL_SR_SELECT_TABLE           = $00001000;
    SQL_SR_UPDATE_TABLE           = $00002000;
    SQL_SR_UPDATE_COLUMN          = $00004000;
  SQL_SQL92_ROW_VALUE_CONSTRUCTOR      = 163;
    // SQL_SQL92_ROW_VALUE_CONSTRUCTOR bitmasks
    SQL_SRVC_VALUE_EXPRESSION = $00000001;
    SQL_SRVC_NULL             = $00000002;
    SQL_SRVC_DEFAULT          = $00000004;
    SQL_SRVC_ROW_SUBQUERY     = $00000008;
  SQL_SQL92_STRING_FUNCTIONS           = 164;
    // SQL_SQL92_STRING_FUNCTIONS
    SQL_SSF_CONVERT       = $00000001;
    SQL_SSF_LOWER         = $00000002;
    SQL_SSF_UPPER         = $00000004;
    SQL_SSF_SUBSTRING     = $00000008;
    SQL_SSF_TRANSLATE     = $00000010;
    SQL_SSF_TRIM_BOTH     = $00000020;
    SQL_SSF_TRIM_LEADING  = $00000040;
    SQL_SSF_TRIM_TRAILING = $00000080;
  SQL_SQL92_VALUE_EXPRESSIONS          = 165;
    // SQL_SQL92_VALUE_EXPRESSIONS bitmasks
    SQL_SVE_CASE     = $00000001;
    SQL_SVE_CAST     = $00000002;
    SQL_SVE_COALESCE = $00000004;
    SQL_SVE_NULLIF   = $00000008;
  SQL_STANDARD_CLI_CONFORMANCE         = 166;
    // SQL_STANDARD_CLI_CONFORMANCE bitmasks
    SQL_SCC_XOPEN_CLI_VERSION1 = $00000001;
    SQL_SCC_ISO92_CLI          = $00000002;
  SQL_STATIC_CURSOR_ATTRIBUTES1        = 167;
  SQL_STATIC_CURSOR_ATTRIBUTES2        = 168;
  SQL_AGGREGATE_FUNCTIONS              = 169;
    // SQL_AGGREGATE_FUNCTIONS bitmasks
    SQL_AF_AVG      = $00000001;
    SQL_AF_COUNT    = $00000002;
    SQL_AF_MAX      = $00000004;
    SQL_AF_MIN      = $00000008;
    SQL_AF_SUM      = $00000010;
    SQL_AF_DISTINCT = $00000020;
    SQL_AF_ALL      = (SQL_AF_AVG or SQL_AF_COUNT or SQL_AF_MAX or SQL_AF_MIN or
                       SQL_AF_SUM or SQL_AF_DISTINCT);
  SQL_DDL_INDEX                        = 170;
    // SQL_DDL_INDEX bitmasks
    SQL_DI_CREATE_INDEX = $00000001;
    SQL_DI_DROP_INDEX   = $00000002;
  SQL_DM_VER                           = 171;
  SQL_INSERT_STATEMENT                 = 172;
    // SQL_INSERT_STATEMENT bitmasks
    SQL_IS_INSERT_LITERALS = 00000001;
    SQL_IS_INSERT_SEARCHED = 00000002;
    SQL_IS_SELECT_INTO     = 00000004;

  SQL_XOPEN_CLI_YEAR                   = 10000;
  SQL_CURSOR_SENSITIVITY               = 10001;
  SQL_DESCRIBE_PARAMETER               = 10002;
  SQL_CATALOG_NAME                     = 10003;
  SQL_COLLATION_SEQ                    = 10004;
  SQL_MAX_IDENTIFIER_LEN               = 10005;
  SQL_ASYNC_MODE                       = 10021;  // new X/Open spec.
    // SQL_ASYNC_MODE values
    SQL_AM_NONE       = 0;
    SQL_AM_CONNECTION = 1;
    SQL_AM_STATEMENT  = 2;
  SQL_MAX_ASYNC_CONCURRENT_STATEMENTS  = 10022;  // new X/Open spec.
  //============================================================================



implementation

// macros...
function SQL_LEN_BINARY_ATTR(length: integer): integer;
begin
  Result := -(length) + SQL_LEN_BINARY_ATTR_OFFSET;
end;

end.
