unit sMain;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, SvcMgr, Dialogs,
  ExtCtrls, IniFiles, uLLFunc, Codebase, ODBC, Sql, messlog, mycbase;

const
  LOG_NAME             = 'sqlsync.log';

  WEIGHT_PATH          = '\\npcs\pcspk\netw\weight\';
  WEIGHT_STRUCT_FILE   = '\\npcs\pcspk\netw\gstru.dbf';
  ACTION_PATH          = '\\npcs\pcspk\netw\action\';
  ACTION_STRUCT_FILE   = '\\npcs\pcspk\netw\astru.dbf';
  QC_FILE              = '\\npcs\pcspk\netw\qc_prod.dbf';
  CARD_FILE            = '\\npcs\ots\card.dbf';

  ODBC_SECTION         = 'MySQL';
  HOST_KEY             = 'Host';
  DATABASE_KEY         = 'DataBase';
  USER_KEY             = 'User';
  PASS_KEY             = 'Key';
  DRIVER_KEY           = 'Driver';
  CUSTOM_KEY           = 'Custom';

type
  TSql2Dbf = class(TService)
    tmrMainProcess: TTimer;
    procedure tmrMainProcessTimer(Sender: TObject);
    procedure ServiceStop(Sender: TService; var Stopped: Boolean);
    procedure ServiceShutdown(Sender: TService);
    procedure ServiceExecute(Sender: TService);
  private
    { Private declarations }
    WorkingPath: string;
    bInProcess: boolean;
    bTerminate: boolean;
    bWeightFile: boolean;
//    bSkipIt: boolean;

//    procedure qcSync;
    function ODBCConnect: boolean;
    function ConnectToSQL(Host, datab, user, pass, Driver, CustomSettings: string): boolean;
    procedure SQLtoDBF(Weight: boolean);

  public
    function GetServiceController: TServiceController; override;
    { Public declarations }
    procedure Process;

  end;

var
  Sql2Dbf: TSql2Dbf;

implementation

{$R *.DFM}

procedure ServiceController(CtrlCode: DWord); stdcall;
begin
  Sql2Dbf.Controller(CtrlCode);
end;

(*
// returns true if there was a message in the queue to process...
function myProcessMessage(var Msg: TMsg; var Terminate: boolean): boolean;
begin
  Result := False;
  if(PeekMessage(Msg, 0, 0, 0, PM_REMOVE))then
  begin
    Result := True;
    if(Msg.Message <> WM_QUIT)then
    begin
      // forward the message on to others...
      TranslateMessage(Msg);
      DispatchMessage(Msg);
    end
    else
      // someone told us to 'quit'!
      Terminate := True;
  end;
end;

// how to do Application.ProcessMessages without a TApplication variable
// will return True if a Quit message is received...
function ProcessMessages: boolean;
var
  Msg: TMsg;
begin
  Result := False;
  // loop until no messages waiting... (myProcessMessage returns False)
  // or WM_QUIT message received (Result = True)
  while(myProcessMessage(Msg, Result))do
    if(Result)then
      break;
end;
*)

function SafeDir(TmpStr: string): string;
begin
  if(Length(TmpStr) > 0)then
    if(TmpStr[Length(TmpStr)] = '\')then
      Delete(TmpStr, Length(TmpStr), 1);
  Result := TmpStr;
end;

function dbfName(cMach: integer; cDate: TDateTime): string;
begin
  Result := Format('%s%.2d', [FormatDateTime('MMDDYY', cDate), cMach]) + '.dbf';
end;

function TSql2Dbf.GetServiceController: TServiceController;
begin
  Result := ServiceController;
end;

procedure TSql2Dbf.ServiceExecute(Sender: TService);
var
  Buffer: array[0..254] of char;
begin
  // windows function to get the full exe name, so we can get the working path from that...
  GetModuleFileName(0, Buffer, SizeOf(Buffer) - 1);
  WorkingPath := SafeDir(ExtractFilePath(Buffer));
{$I-}
  ForceDirectories(WorkingPath + '\log');
{$I+}
  MessageLog := TMessLog.Create(WorkingPath + '\log\' + LOG_NAME, True, True);
  MessageLog.Flush('Starting scan of SQL tables...');

//  bSkipIt := False;
  bWeightFile := True;
  tmrMainProcess.Enabled := True;
  while(not Terminated)do
  begin
    ServiceThread.ProcessRequests(True);

    // need to have a sleep here or this service hogs the CPU...
    Windows.Sleep(100);
  end;
  tmrMainProcess.Enabled := False;

  MessageLog.Free;
end;

(*
// this is no longer called here, done directly by PCS with DBFServer...
procedure TSql2Dbf.qcSync;
var
  d4to: DATA4;
  cb: CODE4;
  stmt, wstmt: TODBCStatement;
  drc, mno, smno, rel, srel: integer;
  rc: boolean;
begin
  wstmt := TODBCStatement.Create(ODBCMain, False);
  stmt := TODBCStatement.Create(ODBCMain);

  cb := code4init;
  code4singleOpen(cb, 0);               // beats me?
  code4safety(cb, 0);                   // allow overwrite?
  code4errFieldName(cb, 0);             // no MessageBox on errors...

  d4to := mopen(cb, QC_FILE, '');
  if(d4to <> nil)then
  begin
    rc := stmt.DoSQL('select * from qc_prod order by machine_no');
    if(rc)then
    begin
      smno := stmt.ifetch('machine_no');
      drc := d4top(d4to);
      while(drc = r4success)do
      begin
        mno := ifetch(d4to, 'machine_no');
  
        // get the two 'files' pointing to the same machno...
        while(smno < mno)do
        begin
          rc := stmt.dbskip(oNext, 1);
          if(rc)then
            smno := stmt.ifetch('machine_no')
          else
            break;
        end;
        while(mno < smno)do
        begin
          drc := d4skip(d4to, 1);
          if(drc = r4success)then
            mno := ifetch(d4to, 'machine_no')
          else
            break;
        end;

        if(rc and (drc = r4success))then
        begin
          // ok, sync 'em up...
          srel := stmt.ifetch('release');
          if(tfetch(d4to, 'release'))then
            rel := 1
          else
            rel := 0;
          if(srel < 0)then
          begin
            MessageLog.Add(Format('Release was -1 for Machine %d', [smno]));
            replace(d4to, 'date', FormatDateTime('YYYYMMDD', stmt.dtfetch('datefield')));
            replace(d4to, 'product', stmt.fetch('product'));
            replace(d4to, 'qc_name', '');
            replace(d4to, 'machine_op', stmt.fetch('machine_op'));
            replace(d4to, 'release', 'F');
            replace(d4to, 'time', stmt.fetch('timefield'));
            wstmt.FormatSQL('update qc_prod set release=0 where machine_no=%d', [smno]);
          end
          else
            if(rel <> srel)then
              if(stmt.ifetch('product') = ifetch(d4to, 'product'))then
              begin
                MessageLog.Add(Format('Release was <> for Machine %d, setting to %d', [smno, rel]));
                wstmt.FormatSQL('update qc_prod set release=%d where machine_no=%d', [rel, smno]);
              end;
          
          drc := d4skip(d4to, 1);
        end;
      end;
    end;
    d4close(d4to);
  end;

  code4close(cb);
  code4initUndo(cb);

  stmt.Free;
  wstmt.Free;
end;
*)

procedure TSql2Dbf.SQLtoDBF(Weight: boolean);
var
  d4to: DATA4;
  cb: CODE4;
  tmpInfo, fieldInfo: PFIELD4INFO;
  stmt, wstmt: TODBCStatement;
  lDate, cDate: TDateTime;
  s, fn, Table, Path, sFile, mWhere, msg: string;
  lMach, cMach, rcvt: integer;
  rc: boolean;
begin
  bInProcess := True;
 try
  if(Weight)then
  begin
    Table := 'weight';
    Path := WEIGHT_PATH;
    sFile := WEIGHT_STRUCT_FILE;
  end
  else
  begin
    Table := 'action';
    Path := ACTION_PATH;
    sFile := ACTION_STRUCT_FILE;
  end;

  wstmt := TODBCStatement.Create(ODBCMain, False);
  stmt := TODBCStatement.Create(ODBCMain);

  rc := stmt.FormatSQL('select * from %s where ifnull(convtd,0)=0 order by machine,dat_e,tim_e,rec_num', [Table]);
  if(rc)then
  begin
    rcvt := 0;
    MessageLog.Flush(Format('Found %s records to convert...', [Table]));

    cb := code4init;
    code4singleOpen(cb, 0);               // beats me?
    code4safety(cb, 0);                   // allow overwrite?
    code4errFieldName(cb, 0);             // no MessageBox on errors...
    d4to := nil;
    fieldInfo := nil;

    lMach := -99;
    lDate := Date -9999;
    while(rc and (not bTerminate))do
    begin
      cMach := stmt.ifetch('machine');
      if(cMach <> lMach)then
      begin
        lMach := cMach;
        lDate := Date - 9999;
      end;

      cDate := stmt.dtfetch('dat_e');
      if(cDate <> lDate)then
      begin
        lDate := cDate;
        if(d4to <> nil)then
        begin
          u4free(PVoid(fieldInfo));
          d4close(d4to);
        end;

        // get the Dated FileName mmddyyMM.dbf  (MM - MachNo)
        fn := dbfName(cMach, cDate);
        MessageLog.Flush(Format('Working on %s', [Path + fn]));
        if(not FileExists(Path + fn))then
        begin
          // if it's not there, create it from the structure file...
          CopyStru(cb, sFile, Path + fn);
          MessageLog.Flush('Created new .dbf: ' + Path + fn);
        end;

        d4to := mopen(cb, Path + fn, '');
        if(d4to = nil)then
          msg := 'Error!'
        else
          msg := 'Ok...';

        MessageLog.Flush(Format('mopen %s - %s', [Path + fn, msg]));
        if(d4to <> nil)then
          fieldInfo := d4fieldInfo(d4to);
      end;

      if(d4to <> nil)then
      begin
        d4appendblank(d4to);
        //MessageLog.Flush('d4appendblank');
        rcvt := rcvt + 1;

        tmpInfo := fieldInfo;
        while(tmpInfo^.name <> nil)do
        begin
          if(d4Field(d4to, PChar(tmpInfo^.name)) <> nil)then
          begin
            if(tmpInfo^.atype = integer('D'))then
              s := FormatDateTime('YYYYMMDD', stmt.dtfetch(tmpInfo^.name))
            else
            begin
              if tmpInfo^.atype = integer('L') then
              begin
                if stmt.tfetch(tmpInfo^.name) then
                   s := 'T'
                else
                   s := 'F';
              end
              else
                s := trim(stmt.fetch(tmpInfo^.name));
            end;
            if(Length(s) > 254)then
              s := Copy(s, 1, 254);
            replace(d4to, tmpInfo^.name, s);
            //MessageLog.Flush(Format('replace %s = %s', [tmpInfo^.name, s]));
          end;
          inc(tmpInfo);
        end;
        mWhere := Format('machine=%d', [cMach]);
        if(cMach = 0)then
          mWhere := '(' + mWhere + ' or machine is null)';
        wstmt.FormatSQL('update %s set convtd=%d where %s and rec_num=%s', [Table, d4recNo(d4to), mWhere, stmt.fetch('rec_num')]);

        //MessageLog.Flush(Format('update %s set convtd=%d where %s and rec_num=%s', [Table, d4recNo(d4to), mWhere, stmt.fetch('rec_num')]));
      end;

      rc := stmt.dbskip(oNext, 1);
    end;
    if(d4to <> nil)then
    begin
      u4free(PVoid(fieldInfo));
      d4close(d4to);
    end;
    code4close(cb);
    code4initUndo(cb);
    MessageLog.Flush(Format('Converted %d records', [rcvt]));
  end;

  stmt.Free;
  wstmt.Free;
 finally
  bInProcess := False;
 end;
end;

function TSql2Dbf.ODBCConnect: boolean;
var
  Ini: TiniFile;
  ODBCHost, ODBCDatab, ODBCUser, ODBCPass, ODBCDriver, ODBCAddSett: string;
begin
  Ini := TiniFile.Create(ExtractFilePath(paramstr(0)) + 'mysqlconn.ini');
  ODBCHost := Ini.ReadString(ODBC_SECTION, HOST_KEY, '');
  ODBCDatab := Ini.ReadString(ODBC_SECTION, DATABASE_KEY, 'nestle');
  ODBCUser := Ini.ReadString(ODBC_SECTION, USER_KEY, '');
  ODBCPass := Ini.ReadString(ODBC_SECTION, PASS_KEY, '');
  ODBCDriver := Ini.ReadString(ODBC_SECTION, DRIVER_KEY, 'MySQL ODBC 3.51 Driver');
  ODBCAddSett := Ini.ReadString(ODBC_SECTION, CUSTOM_KEY, 'SERVER=192.0.0.2');
  Ini.Free;

  Result := ConnectToSQL(ODBCHost, ODBCDatab, ODBCUser, ODBCPass, ODBCDriver, ODBCAddSett);
end;

function TSql2Dbf.ConnectToSQL(Host, datab, user, pass, Driver, CustomSettings: string): boolean;
var
  err: boolean;
begin
  ODBCmain := TODBC.Create;
  if(ODBCmain <> nil)then
  begin
    ODBCMain.Mess := MessageLog;
    ODBCmain.DoShowerrors := True;
    // and 'connect' to the the ODBC system...
    err := not ODBCmain.ConnectToDataSource(Host, user, pass, datab, False, Driver, CustomSettings);
    //ODBCmain.BufferRecs := True;
  end
  else
    err := True;

  if(err)then
  begin
    //MessageDlg('Error Connecting to SQL Data!', mtError, [mbOk], 0);
    if(odbcmain <> nil)then
      odbcmain.free;
    odbcmain := nil;
  end;
  Result := not err;
end;

procedure TSql2Dbf.Process;
var
  Host, datab, user, pass: string;
begin
 try
  if(ODBCConnect)then
  begin
    // WE DON'T NEED TO DO ANYTHING ONCE A MINUTE, SO I JUST
    // CHANGED THE INTERVAL TO 2 MINUTES...

    // we only do this every other time here (every other minute)
    //if(not bSkipIt)then
    //begin
      //MessageLog.Add('Connected');
      SQLtoDBF(bWeightFile);  // True = weight file  False = action file...
      bWeightFile := not bWeightFile;

      //SQLtoDBF(False); // action file
      // and the qc_prod file...
      // qcSync;  done in pcs with dbfserver...
    //end;

    //bSkipIt := not bSkipIt;
  end
  else
    MessageLog.Add(Format('ERROR - H:%s D:%s U:%s P:%s', [Host, datab, user, pass]));

  if(ODBCMain <> nil)then
    ODBCMain.Free;
  ODBCMain := nil;

 finally
  MessageLog.Flush;
 end;
end;

function GetFoxTime(t: integer): TDateTime;
begin
  Result := EncodeTime(t div 100, t mod 100, 0, 0);
end;

procedure TSql2Dbf.tmrMainProcessTimer(Sender: TObject);
begin
  tmrMainProcess.Enabled := False;
 try
  Process;
 finally
  tmrMainProcess.Interval := 120000;
  tmrMainProcess.Enabled := True;
 end;
end;

procedure TSql2Dbf.ServiceStop(Sender: TService; var Stopped: Boolean);
begin
  MessageLog.Add('Stop request received!');

  bTerminate := True;
  while(bInProcess)do
  begin
    uLLFunc.ProcessMessages;
    Windows.Sleep(50);
  end;
  MessageLog.Add('Process safely stopped...');

  Stopped := True;
end;

procedure TSql2Dbf.ServiceShutdown(Sender: TService);
begin
  MessageLog.Add('Shutdown request received!');

  bTerminate := True;
  while(bInProcess)do
  begin
    uLLFunc.ProcessMessages;
    Windows.Sleep(50);
  end;
  MessageLog.Add('Process safely shutdown...');
end;

end.
