unit uLLFunc;

interface

uses
  Windows, Messages, Math;

const
  CHECKS_PER_SECOND = 10;   //

var
  myllTimer: boolean;

procedure mySleep(ms: integer);
function ProcessMessages: boolean;

implementation

// returns true if there was a message in the queue to process...
function myProcessMessage(var Msg: TMsg; var Terminate: boolean): boolean;
begin
  Result := False;
  if(PeekMessage(Msg, 0, 0, 0, PM_REMOVE))then
  begin
    Result := True;
    if(Msg.Message <> WM_QUIT)then
    begin
      // forward the message on to others...
      TranslateMessage(Msg);
      DispatchMessage(Msg);
    end
    else
      // someone told us to 'quit'!
      Terminate := True;
  end;
end;

// how to do Application.ProcessMessages without a TApplication variable
// will return True if a Quit message is received...
function ProcessMessages: boolean;
var
  Msg: TMsg;
begin
  Result := False;
  // loop until no messages waiting... (myProcessMessage returns False)
  // or WM_QUIT message received (Result = True)
  while(myProcessMessage(Msg, Result))do
    if(Result)then
      break;
end;

procedure myTimerProc(Wnd: THandle; Msg, TimerID, SysTime: Longint); stdcall;
begin
  // this is to get control back to our thread below, eventually...
  myllTimer := True;
end;

// this 'Sleep' lets other processes have full control to allow them to
// do 'stuff', while we just wait around....
procedure mySleep(ms: integer);
var
  hdl: word;
  i: integer;
  Quit: boolean;
begin
  myllTimer := False;
  // if you don't use this SetTimer, the WaitMessage call may NEVER RETURN!
  hdl := SetTimer(0, 0, Min(ms, Trunc(1000 / CHECKS_PER_SECOND)), @myTimerProc);
  if(hdl <> 0)then
  begin
    Quit := False;
    // waits for so many milliseconds, doing 'CHECKS_PER_SECOND' ProcessMessages/WaitMessage per second
    for i := 1 to Max(Trunc(ms / (Trunc(1000 / CHECKS_PER_SECOND))), 1) do
    begin
      // some other event may cause WaitMessage to return, not myTimerProc,
      // in which case we want to go back into WaitMessage, until it IS myTimerProc...
      while(not myllTimer)do
      begin
        // MUST call ProcessMessages here or myTimerProc won't ever get called...
        // also, it clears out all messages in our queue, so
        // WaitMessage below can do it's 'work'...
        // returns true if WM_QUIT received, so break out then!
        Quit := ProcessMessages;
        if(Quit)then
          break;

        if(not myllTimer)then
          WaitMessage;   // this message handler is how you let other threads have control...
        // here's the API help on it:
        // The WaitMessage function yields control to other threads when
        // a thread has no other messages in its message queue.
        // The WaitMessage function suspends the thread and does not
        // return until a new message is placed in the thread's message queue.
      end;
      myllTimer := False;
      if(Quit)then
        break;
    end;
    KillTimer(0, hdl);
  end;
end;

end.
